"""
Classifies: CHEBI:24866 salt
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_salt(smiles: str):
    """
    Determines if a molecule is a salt (assembly of cations and anions)
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a salt, False otherwise
        str: Reason for classification
    """
    try:
        mol = Chem.MolFromSmiles(smiles)
        if mol is None:
            return False, "Invalid SMILES string"
            
        # Split into fragments
        fragments = Chem.GetMolFrags(mol, asMols=True)
        
        if len(fragments) < 1:
            return False, "No molecular fragments found"
            
        # Check formal charges
        cations = []
        anions = []
        neutral = []
        
        # Track total system charge
        total_system_charge = 0
        
        for frag in fragments:
            total_charge = sum(atom.GetFormalCharge() for atom in frag.GetAtoms())
            total_system_charge += total_charge
            
            # Check for zwitterions
            has_positive = any(atom.GetFormalCharge() > 0 for atom in frag.GetAtoms())
            has_negative = any(atom.GetFormalCharge() < 0 for atom in frag.GetAtoms())
            
            if total_charge > 0 or (has_positive and not has_negative):
                cations.append(frag)
            elif total_charge < 0 or (has_negative and not has_positive):
                anions.append(frag)
            elif has_positive and has_negative:
                # Zwitterion - treat as neutral
                neutral.append(frag)
            else:
                neutral.append(frag)

        # Special cases
        if len(fragments) == 1:
            if len(cations) == 0 and len(anions) == 0:
                return False, "Single neutral molecule"
            elif len(cations) > 0 and len(anions) > 0:
                return False, "Single zwitterionic molecule"

        # Check for acid/base pairs
        if len(neutral) > 0 and (len(cations) == 0 or len(anions) == 0):
            # Look for acid/base characteristics in neutral fragments
            for frag in neutral:
                for atom in frag.GetAtoms():
                    if atom.GetSymbol() in ['N', 'O'] and atom.GetTotalNumHs() > 0:
                        # Potential acid/base participant
                        if len(cations) == 0:
                            cations.append(frag)
                        elif len(anions) == 0:
                            anions.append(frag)

        # Final classification
        if len(cations) > 0 and len(anions) > 0:
            msg = f"Salt containing {len(cations)} cation(s) and {len(anions)} anion(s)"
            if len(neutral) > 0:
                msg += f" and {len(neutral)} neutral species"
            return True, msg
        elif len(cations) == 0 and len(anions) == 0:
            return False, "No charged species present"
        elif len(cations) == 0:
            return False, "Only anions present - not a salt"
        else:
            return False, "Only cations present - not a salt"
                
    except Exception as e:
        return None, f"Error analyzing structure: {str(e)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24866',
                          'name': 'salt',
                          'definition': 'A salt is an assembly of cations and '
                                        'anions.',
                          'parents': ['CHEBI:37577']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.5941644562334217 is too low.\n'
               'True positives: '
               "[('[I-].C[N+]1(C)CCC(CC1)OC(=O)C(c1ccccc1)c1ccccc1', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[Na+].[Na+].Oc1cc(c2ccc3c(cc(c4ccc1c2c34)S([O-])(=O)=O)S([O-])(=O)=O)S([O-])(=O)=O', "
               "'Salt containing 3 cation(s) and 1 anion(s)'), "
               "('[O-]Cl(=O)(=O)=O.[H]C(=C([H])c1oc2ccccc2[n+]1CCCCCCCCCCCCCCCCCC)C([H])=C1Oc2ccccc2N1CCCCCCCCCCCCCCCCCC', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[K+].C1=CC=CC2=C1C(=C(C(O2)=O)[C@@H](CC(C)=O)C3=CC=CC=C3)[O-]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[S--].[Fe+3].[As-]', 'Salt containing 1 cation(s) and 2 "
               "anion(s)'), "
               "('[Na+].C=1(C(=CC=CC1)NC=2C=CC(=CC2)\\\\C(=C/3\\\\C=C/C(/C=C3)=N/C=4C=CC=CC4)\\\\C=5C=C(C(=CC5)N)C)S([O-])(=O)=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('C1(C(C[NH2+]CCCC)O)=CC=C(C=C1)O.S(=O)(=O)([O-])[O-].C1(C(C[NH2+]CCCC)O)=CC=C(C=C1)O', "
               "'Salt containing 2 cation(s) and 1 anion(s)'), "
               "('[O]N1C(C([N+](=C1C=2C=CC(=CC2)C(=O)[O-])[O-])(C)C)(C)C.[K+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[Na+].[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)C(C([O-])=O)c1ccccc1)C([O-])=O', "
               "'Salt containing 2 cation(s) and 1 anion(s)'), "
               "('[Cl-].[Cl-].[H][C@]12C[C@@]3([H])[C@@]4(CC[N@@+]3(CC=C)C\\\\C1=C\\\\CO)c1ccccc1N1\\\\C=C3\\\\[C@@]5([H])C[C@@]6([H])[C@@]7(CC[N@@+]6(CC=C)C\\\\C5=C\\\\CO)c5ccccc5N(\\\\C=C2/[C@@]41[H])[C@@]37[H]', "
               "'Salt containing 1 cation(s) and 2 anion(s)'), "
               "('[H+].[H+].O[C@H]([C@@H](O)C([O-])=O)C([O-])=O.C[C@H](N)[C@H](O)c1cccc(O)c1', "
               "'Salt containing 2 cation(s) and 1 anion(s) and 1 neutral "
               "species'), ('[Co++].CC([O-])=O.CC([O-])=O', 'Salt containing 1 "
               "cation(s) and 2 anion(s)'), "
               "('[Na+].[Na+].[O-]C(=O)CC([O-])=O', 'Salt containing 2 "
               "cation(s) and 1 anion(s)'), "
               "('[K+].CC[N+]1=C(/C=C/C=C/C=C2\\\\N(C)c3ccc(NC(=O)CI)cc3C2(C)C)C(C)(C)c2c1ccc1c(cc(cc21)S([O-])(=O)=O)S([O-])(=O)=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[Na+].[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@]2(NC(=O)C([H])(C([O-])=O)c1ccsc1)OC)C([O-])=O', "
               "'Salt containing 2 cation(s) and 1 anion(s)'), "
               "('[Na+].[Na+].[Na+].OC(=O)CN(CCN(CC([O-])=O)CC([O-])=O)CC([O-])=O', "
               "'Salt containing 3 cation(s) and 1 anion(s)'), "
               "('O.O.O.O.O.[Mn+2].S([O-])([O-])(=O)=O', 'Salt containing 1 "
               "cation(s) and 1 anion(s) and 5 neutral species'), "
               "('[K+].[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2N1C(=O)[C@H](NC1(C)C)c1ccccc1)C([O-])=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].COC1=CC(OC)=NC(OC2=CC=CC(OC3=NC(OC)=CC(OC)=N3)=C2C([O-])=O)=N1', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[H]C(=C([H])c1cc([o+]c2cc3N(CC)C(C)(C)C=C(C)c3cc12)C(C)(C)C)C([H])=C1N(CCCS([O-])(=O)=O)c2ccc(cc2C1(C)CCCC(O)=O)S([O-])(=O)=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[Na+].[Ca++].[O-]C(=O)CN(CCN(CC([O-])=O)CC([O-])=O)CC([O-])=O', "
               "'Salt containing 3 cation(s) and 1 anion(s)'), "
               "('[As-](F)(F)(F)(F)(F)F.[Na+]', 'Salt containing 1 cation(s) "
               "and 1 anion(s)'), ('[Be++].[O-]S([O-])(=O)=O', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)[C@H](NC(=O)N1CCN(C1=O)S(C)(=O)=O)c1ccccc1)C([O-])=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[H+].[H+].[O-]C(=O)\\\\C=C/C([O-])=O.CCCCCNC(=N)N\\\\N=C\\\\c1c[nH]c2ccc(OC)cc12', "
               "'Salt containing 2 cation(s) and 1 anion(s) and 1 neutral "
               "species'), ('[Cl-].[Cl-].[Ca++]', 'Salt containing 1 cation(s) "
               "and 2 anion(s)'), "
               "('[Na+].[Na+].[O-][Cr](=O)(=O)O[Cr]([O-])(=O)=O', 'Salt "
               "containing 2 cation(s) and 1 anion(s)'), "
               "('[Cr+3].[Cr+3].[O-]S([O-])(=O)=O.[O-]S([O-])(=O)=O.[O-]S([O-])(=O)=O', "
               "'Salt containing 2 cation(s) and 3 anion(s)'), "
               "('[H+].[Cl-].NC1=NCC2N1c1ccccc1Cc1ccccc21', 'Salt containing 1 "
               "cation(s) and 1 anion(s) and 1 neutral species'), "
               "('[Cl-].[H][N+](C)(C)CCCN1c2ccccc2CCc2ccccc12', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('[Cl-].[H][C@@]1(C[C@]23CC[C@]1(OC)[C@@H]1Oc4c(O)ccc5C[C@H]2[NH+](CC[C@@]31c45)CC1CC1)[C@](C)(O)C(C)(C)C', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[K+].O(C(C)C([O-])=O)CCOCC', 'Salt containing 1 cation(s) "
               "and 1 anion(s)'), "
               "('[O-]S([O-])(=O)=O.[O-]S([O-])(=O)=O.[O-]S([O-])(=O)=O.[O-]S([O-])(=O)=O.[O-]S([O-])(=O)=O.CC(C)CCCCC(=O)N[C@@H](CC[NH3+])C(=O)N[C@@H]([C@@H](C)O)C(=O)N[C@@H](CC[NH3+])C(=O)N[C@H]1CCNC(=O)[C@@H](NC(=O)[C@H](CC[NH3+])NC(=O)[C@H](CC[NH3+])NC(=O)[C@H](CC(C)C)NC(=O)[C@@H](CC(C)C)NC(=O)[C@H](CC[NH3+])NC1=O)[C@@H](C)O.CC(C)CCCCC(=O)N[C@@H](CC[NH3+])C(=O)N[C@@H]([C@@H](C)O)C(=O)N[C@@H](CC[NH3+])C(=O)N[C@H]1CCNC(=O)[C@@H](NC(=O)[C@H](CC[NH3+])NC(=O)[C@H](CC[NH3+])NC(=O)[C@H](CC(C)C)NC(=O)[C@@H](CC(C)C)NC(=O)[C@H](CC[NH3+])NC1=O)[C@@H](C)O', "
               "'Salt containing 2 cation(s) and 5 anion(s)'), "
               "('[Na+].[Na+].[H][C@@]12CCC3=CC(=O)CC[C@]3(C)[C@@]1([H])[C@@H](O)C[C@@]1(C)[C@@]2([H])CC[C@]1(O)C(=O)COP([O-])([O-])=O', "
               "'Salt containing 2 cation(s) and 1 anion(s)'), "
               "('[Ca++].CC(=O)NCCCS([O-])(=O)=O.CC(=O)NCCCS([O-])(=O)=O', "
               "'Salt containing 1 cation(s) and 2 anion(s)'), "
               "('O.O.O.O.[Ca++].[O-][N+]([O-])=O.[O-][N+]([O-])=O', 'Salt "
               "containing 1 cation(s) and 2 anion(s) and 4 neutral species'), "
               "('[O-]S([O-])(=O)=O.CCNc1cc2[o+]c3cc(ccc3nc2c2ccccc12)N(CC)CC.CCNc1cc2[o+]c3cc(ccc3nc2c2ccccc12)N(CC)CC', "
               "'Salt containing 2 cation(s) and 1 anion(s)'), "
               "('[Na+].[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)Cc1ccccc1)C([O-])=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[H+].[H+].CS([O-])(=O)=O.CS([O-])(=O)=O.CN1CCN(CCCN2c3ccccc3Sc3ccc(Cl)cc23)CC1', "
               "'Salt containing 2 cation(s) and 2 anion(s) and 1 neutral "
               "species'), ('S(S([O-])=O)([O-])(=O)=O.[Na+].[Na+]', 'Salt "
               "containing 2 cation(s) and 1 anion(s)'), "
               "('[H+].[Cl-].CCN(CC)CCNC(=O)c1ccc(N)cc1', 'Salt containing 1 "
               "cation(s) and 1 anion(s) and 1 neutral species'), "
               "('[Ca++].[H][C@@]1(C[C@H]2CO[C@@H](C\\\\C(C)=C\\\\C(=O)OCCCCCCCCC([O-])=O)[C@H](O)[C@@H]2O)O[C@@]1([H])[C@@H](C)[C@H](C)O.[H][C@@]1(C[C@H]2CO[C@@H](C\\\\C(C)=C\\\\C(=O)OCCCCCCCCC([O-])=O)[C@H](O)[C@@H]2O)O[C@@]1([H])[C@@H](C)[C@H](C)O', "
               "'Salt containing 1 cation(s) and 2 anion(s)'), "
               "('[Na+].[H][C@]1(C[NH2+][C@@]([H])(C1)C(=O)Nc1cccc(c1)C([O-])=O)SC1=C(N2C(=O)[C@]([H])([C@@H](C)O)[C@@]2([H])[C@H]1C)C([O-])=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[NH4+].[NH4+].[S-]C(=S)NCCNC([S-])=S', 'Salt containing 2 "
               "cation(s) and 1 anion(s)'), "
               "('[Al+3].[Al+3].[O-]S([O-])(=O)=O.[O-]S([O-])(=O)=O.[O-]S([O-])(=O)=O', "
               "'Salt containing 2 cation(s) and 3 anion(s)'), "
               "('[Na+].[O-]B=O', 'Salt containing 1 cation(s) and 1 "
               "anion(s)'), "
               "('[I-].[I-].[I-].[I-].[H]C(=CC([H])=C1Oc2ccccc2N1C)c1cc[n+](CCC(CCCC(CC[n+]2ccc(C([H])=CC([H])=C3Oc4ccccc4N3C)c3ccccc23)=[N+](C)C)=[N+](C)C)c2ccccc12', "
               "'Salt containing 1 cation(s) and 4 anion(s)'), "
               "('[Na+].CC(=O)N[C@H]1[C@H](O)O[C@H](CO)[C@@H](O)[C@@H]1O[C@@H]1O[C@H](COS([O-])(=O)=O)[C@@H](O)[C@H](O)[C@H]1O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[NH4+].[NH4+].[Ni++].[O-]S([O-])(=O)=O.[O-]S([O-])(=O)=O', "
               "'Salt containing 3 cation(s) and 2 anion(s)'), "
               "('C1[C@@]2([C@]3(CC[C@]4([C@]([C@@]3([C@@H](C[C@@]2(C[C@@H](C1)O)[H])O)[H])(CC[C@@]4([C@@H](CCCS([O-])(=O)=O)C)[H])[H])C)[H])C.[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[Na+].[Na+].[Na+].[Na+].[Na+].[Na+].[Na+].[Na+].[Na+].CO[C@H]1O[C@H](COS([O-])(=O)=O)[C@@H](O[C@@H]2O[C@H]([C@@H](O[C@H]3O[C@H](COS([O-])(=O)=O)[C@@H](O[C@@H]4O[C@@H]([C@@H](O[C@H]5O[C@H](COS([O-])(=O)=O)[C@@H](O)[C@H](O)[C@H]5NS([O-])(=O)=O)[C@H](O)[C@H]4O)C(O)=O)[C@H](OS([O-])(=O)=O)[C@H]3NS([O-])(=O)=O)[C@H](O)[C@H]2OS([O-])(=O)=O)C(O)=O)[C@H](O)[C@H]1NS([O-])(=O)=O', "
               "'Salt containing 10 cation(s) and 1 anion(s)'), "
               "('[Na+].[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)c1c(C)onc1-c1c(Cl)cccc1Cl)C([O-])=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('O.[Mn++].[O-]S([O-])(=O)=O', 'Salt containing 1 cation(s) "
               "and 1 anion(s) and 1 neutral species'), "
               "('[Na+].[Na+].[O-]S([O-])(=O)=O', 'Salt containing 2 cation(s) "
               "and 1 anion(s)'), "
               "('[Na+].[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)[C@]([H])(NC(=O)c1cnc2cccnc2c1O)c1ccccc1)C([O-])=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Ca++].CCCCCCCCCCCCCCCCCC([O-])=O.CCCCCCCCCCCCCCCCCC([O-])=O', "
               "'Salt containing 1 cation(s) and 2 anion(s)'), "
               "('[Na+].[Na+].N[C@@H](Cc1ccc([O-])cc1)C([O-])=O', 'Salt "
               "containing 2 cation(s) and 1 anion(s)'), "
               "('[Cl-].CCC[NH2+]C(C)C(=O)Nc1ccccc1C', 'Salt containing 1 "
               "cation(s) and 1 anion(s)'), "
               "('[Na+].[Na+].Cl[Pt--](Cl)(Cl)(Cl)(Cl)Cl', 'Salt containing 2 "
               "cation(s) and 1 anion(s)'), ('[Li+].F[P-](F)(F)(F)(F)F', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('[Zn++].[O-][N+]([O-])=O.[O-][N+]([O-])=O', 'Salt containing "
               "1 cation(s) and 2 anion(s)'), ('Cl[O-].[Ca+2].Cl[O-]', 'Salt "
               "containing 1 cation(s) and 2 anion(s)'), "
               "('[Cl-].CC(C)([NH3+])Cc1ccccc1', 'Salt containing 1 cation(s) "
               "and 1 anion(s)'), "
               "('[H+].[H+].[H+].[H+].[O-]C(=O)\\\\C=C/C([O-])=O.[O-]C(=O)\\\\C=C/C([O-])=O.CN1CCN(CC1)c1nc2cc(ccc2n2cccc12)C(F)(F)F', "
               "'Salt containing 4 cation(s) and 2 anion(s) and 1 neutral "
               "species'), "
               "('[Na+].[Na+].[H][C@@]12C[C@H](C)[C@](O)(C(=O)COP([O-])([O-])=O)[C@@]1(C)C[C@H](O)[C@@]1(F)[C@@]2([H])CCC2=CC(=O)C=C[C@]12C', "
               "'Salt containing 2 cation(s) and 1 anion(s)'), "
               "('[Cl-].CN(C)c1ccc2c(-c3ccccc3C(O)=O)c3ccc(cc3[o+]c2c1)N(C)C', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('O.O.O.O.O.O.O.[Cr+3].[O-][N+]([O-])=O.[O-][N+]([O-])=O.[O-][N+]([O-])=O', "
               "'Salt containing 1 cation(s) and 3 anion(s) and 7 neutral "
               "species'), ('O.[Na+].CC(=O)[N-]S(=O)(=O)c1ccc(N)cc1', 'Salt "
               "containing 1 cation(s) and 1 anion(s) and 1 neutral species'), "
               "('[H+].[H+].[Cl-].[Cl-].COc1ccc2nc3cc(Cl)ccc3c(NC(C)CCCN(CCCl)CCCl)c2c1', "
               "'Salt containing 2 cation(s) and 2 anion(s) and 1 neutral "
               "species'), "
               "('[Na+].[H][C@]1(O[C@H](C)[C@@H](O[C@]2([H])O[C@H](CO)[C@@H](O)[C@H](O[C@]3([H])O[C@H](CO)[C@@H](O)[C@H](OC)[C@H]3O)[C@H]2O)[C@H](O)[C@H]1O)O[C@@H]1[C@@H](O)[C@@H](CO[C@@]1([H])O[C@H]1CC[C@]2(C)C3=C[C@H](O)C45C(=O)O[C@@](C)(C(=O)CCC(C)(C)O)[C@@]4(O)CC[C@@]5(C)[C@]3([H])CC[C@@]2([H])C1(C)C)OS([O-])(=O)=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('O(CC(CCCCCC)CCCC)S([O-])(=O)=O.[Na+]', 'Salt containing 1 "
               "cation(s) and 1 anion(s)'), "
               "('[Cl-].Oc1cc(O)c2ccc([o+]c2c1)-c1ccc(O)c(O)c1', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('[H+].[H+].[H+].[H+].[O-]C(=O)\\\\C=C/C([O-])=O.[O-]C(=O)\\\\C=C/C([O-])=O.CCSc1ccc2Sc3ccccc3N(CCCN3CCN(C)CC3)c2c1', "
               "'Salt containing 4 cation(s) and 2 anion(s) and 1 neutral "
               "species'), ('[K+].[I-]', 'Salt containing 1 cation(s) and 1 "
               "anion(s)'), "
               "('[Na+].C1=CC=CC2=C1C(=C(C(O2)=O)[C@H](CC(C)=O)C3=CC=CC=C3)[O-]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Br-].CC[n+]1c(-c2ccccc2)c2cc(N)ccc2c2ccc(N)cc12', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), ('[Na+].[O-]C(=O)CF', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('Clc1ccc(Cn2c(C[NH+]3CCCC3)nc3ccccc23)cc1.[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)Cc1ccccc1)C([O-])=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Li+].[Li+].Nc1ccc2c(-c3cc(ccc3C([O-])=O)C(O)=O)c3ccc(N)c(c3[o+]c2c1S([O-])(=O)=O)S([O-])(=O)=O', "
               "'Salt containing 2 cation(s) and 1 anion(s)'), "
               "('[Na+].CC(C)(O)c1ccccc1CC[C@@H](SCC1(CC1)CC([O-])=O)c1cccc(\\\\C=C\\\\c2ccc3ccc(Cl)cc3n2)c1', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[Na+].[Na+].CC(\\\\C=C\\\\C=C1\\\\N(CCS([O-])(=O)=O)c2ccc(cc2C1(C)C)S([O-])(=O)=O)=C/C=C/C1=[N+](CCS([O-])(=O)=O)c2ccc(cc2C1(C)C)S([O-])(=O)=O', "
               "'Salt containing 3 cation(s) and 1 anion(s)'), "
               "('[Cl-].CCCCC(O)(C(=O)NCCCCCC[NH3+])C(=O)N(Nc1ccccc1)c1ccccc1', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[Na+].[O-][Se]([O-])=O', 'Salt containing 2 cation(s) "
               "and 1 anion(s)'), ('C(C(=O)[O-])(=O)[O-].[Na+].[Na+]', 'Salt "
               "containing 2 cation(s) and 1 anion(s)'), "
               "('[H+].[H+].[O-]C(=O)\\\\C=C\\\\C([O-])=O.OCCOCCN1CCN(CC1)C1=Nc2ccccc2Sc2ccccc12.OCCOCCN1CCN(CC1)C1=Nc2ccccc2Sc2ccccc12', "
               "'Salt containing 2 cation(s) and 1 anion(s) and 2 neutral "
               "species'), ('O.O.O=[V++].[O-]S([O-])(=O)=O', 'Salt containing "
               "1 cation(s) and 1 anion(s) and 2 neutral species'), "
               "('[Na+].OC[C@H](N=[N+]=[N-])[C@H]1O[C@@](C[C@@H](O)[C@H]1O)(OCC=C)C([O-])=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Br-].CCCCCCCCCC[N+](C)(C)C', 'Salt containing 1 cation(s) "
               "and 1 anion(s)'), "
               "('[H+].[Cl-].CN\\\\C(NCCSCc1ccc(CN(C)C)o1)=C/[N+]([O-])=O', "
               "'Salt containing 1 cation(s) and 1 anion(s) and 1 neutral "
               "species'), ('[K+].[O-]Br(=O)=O', 'Salt containing 1 cation(s) "
               "and 1 anion(s)'), ('[F-].CCCC[N+](CCCC)(CCCC)CCCC', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('[Mg++].CC([O-])=O.CC([O-])=O', 'Salt containing 1 cation(s) "
               "and 2 anion(s)'), "
               "('[Cl-].ClCc1ccc(cc1)C1=c2cc3CCC[N+]4=c3c(CCC4)c2Oc2c3CCCN4CCCc(cc12)c34', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[Na+].O[B-]1(O)OO[B-](O)(O)OO1', 'Salt containing 2 "
               "cation(s) and 1 anion(s)'), "
               "('O.O.O.O.O.O.O.[Na+].[Na+].Nc1nc(=O)c2c(CCc3ccc(cc3)C(=O)N[C@@H](CCC([O-])=O)C([O-])=O)c[nH]c2[nH]1', "
               "'Salt containing 2 cation(s) and 1 anion(s) and 7 neutral "
               "species'), "
               "('[Na+].[Na+].[Na+].[Na+].Cc1cc(ccc1\\\\N=N\\\\c1c(O)c2c(N)cc(cc2cc1S([O-])(=O)=O)S([O-])(=O)=O)-c1ccc(\\\\N=N\\\\c2c(O)c3c(N)cc(cc3cc2S([O-])(=O)=O)S([O-])(=O)=O)c(C)c1', "
               "'Salt containing 4 cation(s) and 1 anion(s)'), "
               "('[K+].[K+].[K+].[K+].[K+].[K+].[O-]C(=O)CN(CC([O-])=O)c1ccccc1OCCOc1cc(NC(=O)c2ccc3c(c2)C(=O)OC32c3cc(Cl)c([O-])cc3Oc3cc([O-])c(Cl)cc23)ccc1N(CC([O-])=O)CC([O-])=O', "
               "'Salt containing 6 cation(s) and 1 anion(s)'), "
               "('S([O-])([O-])(=O)=O.S([O-])([O-])(=O)=O.[Al+3].O.O.O.O.O.O.O.O.O.O.O.O.[NH4+]', "
               "'Salt containing 2 cation(s) and 2 anion(s) and 12 neutral "
               "species'), "
               "('[Cu++].[H]O[H].[H]O[H].[H]O[H].[H]O[H].[H]O[H].[O-]S([O-])(=O)=O', "
               "'Salt containing 1 cation(s) and 1 anion(s) and 5 neutral "
               "species'), "
               "('[Na+].[Na+].[H][C@@]12CCC3C[C@H](OS([O-])(=O)=O)[C@H](C[C@]3(C)[C@@]1([H])CC[C@]1(C)[C@@]([H])([C@H](C)C(OC(=O)CCC)C(OC(=O)CCC)C(CC)C(C)C)[C@@H](O)[C@H](OC(C)=O)[C@@]21[H])OS([O-])(=O)=O', "
               "'Salt containing 2 cation(s) and 1 anion(s)')]\n"
               'False positives: '
               "[('O.[Na+].COc1c(cc(cc1C(C)(C)C)-n1ccc(=O)[nH]c1=O)-c1ccc2cc([N-]S(C)(=O)=O)ccc2c1', "
               "'Salt containing 1 cation(s) and 1 anion(s) and 1 neutral "
               "species'), ('P(=O)([O-])([O-])C1OC1C.[Na+].[Na+]', 'Salt "
               "containing 2 cation(s) and 1 anion(s)'), "
               "('[H+].[H+].[H+].[H+].[H+].[H+].[Ga+3].CC(C)(C)[Si]([18F])(C1=CC=C(C=C1)C(=O)NC[C@@H](NC(=O)CC[C@H](N1CCN(CC([O-])=O)CCN(CC([O-])=O)CCN(CC([O-])=O)CC1)C([O-])=O)C(=O)N[C@H](CCCCNC(=O)CCC(=O)NCCC[C@@H](NC(=O)CC[C@H](NC(=O)N[C@@H](CCC([O-])=O)C([O-])=O)C([O-])=O)C([O-])=O)C([O-])=O)C(C)(C)C', "
               "'Salt containing 7 cation(s) and 1 anion(s)'), "
               "('[C@]1(C(N=C(N1)C2=C(C=CC=N2)C(=O)[O-])=O)(C)C(C)C.C(C)([NH3+])C', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[O-]Cl(=O)(=O)=O.CN(C)c1ccc2cc3ccc(cc3[n+](CCCC(=O)ON3C(=O)CCC3=O)c2c1)N(C)C', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('O.O.O.O.[Na+].[Na+].OP([O-])(=O)C(Cl)(Cl)P(O)([O-])=O', "
               "'Salt containing 2 cation(s) and 1 anion(s) and 4 neutral "
               "species'), "
               "('[O-]C(=O)C(F)(F)F.[O-]C(=O)C(F)(F)F.NC(=[NH2+])NCCCc1[nH+]ccc2c3ccc(Br)cc3[nH]c12', "
               "'Salt containing 1 cation(s) and 2 anion(s)'), "
               "('[Zn+2].S1C(=N[C@](C1)(C(=O)[O-])C)C([C@@H](O)[C@H]2N([C@@H]([C@@H]3N=C(C4=C([O-])C=CC=C4CCCCC)SC3)SC2)C)(C)C', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Gd+3].COCCNC(=O)CN(CCN(CCN(CC([O-])=O)CC(=O)NCCOC)CC([O-])=O)CC([O-])=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('CC[NH+](CC)CC.NC1=C(Cl)C(=NC(=N1)C1CC1)C([O-])=O', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('C=1(C2=C(N=CN1)N(C=N2)[C@H]3[C@H](OC(CCC)=O)[C@H]4[C@@H](COP(O4)(=O)[O-])O3)NC(=O)CCC.[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('C=1(C(=CC=2C=CC=CC2N1)C([O-])=O)C=3NC([C@@](N3)(C)C(C)C)=O.[NH4+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), ('[OH-].[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Br-].CC[N+](CCOC(C(C1=CC=CC=C1)(C=2C=CC=CC2)O)=O)(CC)C', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[K+].[K+].C[C@]12CC[C@@](C)(C[C@H]1C1=CC(=O)[C@@H]3[C@@]4(C)CC[C@H](O[C@H]5O[C@@H]([C@@H](O)[C@H](O)[C@H]5O[C@@H]5O[C@@H]([C@@H](O)[C@H](O)[C@H]5O)C([O-])=O)C([O-])=O)C(C)(C)[C@@H]4CC[C@@]3(C)[C@]1(C)CC2)C(O)=O', "
               "'Salt containing 2 cation(s) and 1 anion(s)'), "
               "('[I-].[I-].C=12[C@@]3([N+](C)(CCC1C=C(C(=C2OC4=CC=C(C=C4)C[C@@]5([N+](CCC6=C5C=C(OC=7C(OC)=CC=C(C3)C7)C(=C6)OC)(C)C)[H])OC)OC)C)[H]', "
               "'Salt containing 1 cation(s) and 2 anion(s)'), "
               "('[Ca+2].O=C(C=C([O-])C(CC(CC=CC(C(O)C(C(O)CC1OC(C2OC(C(O)C)(C)CC2)(C)CC1)C)C)C)C)C(CC(CC(CCC(=O)[O-])C)C)C', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('CC[NH3+].CC([O-])=O', 'Salt containing 1 cation(s) and 1 "
               "anion(s)'), ('[Ag+].Nc1ccc(cc1)S(=O)(=O)[N-]c1ncccn1', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('[C@H]1(O[C@@H]([C@@H]([C@@H]([C@H]1O)O[C@@]23O[C@]([C@@H]([C@H](C2)O)NC3=O)([C@@H]([C@@H](CO)O)O)[H])O)CO)O[C@H]4[C@@H]([C@H]([C@@H](O[C@@H]4COS(=O)(=O)[O-])O[C@@H]5[C@H]([C@@H](O[C@@H]([C@@H]5O)CO)O[C@H]6[C@@H]([C@H]([C@@H](O[C@@H]6CO)OC[C@@H](CCCCCCCCCCCCCC)CCCCCCCCCCCCCC)O)O)O)NC(=O)C)O[C@@H]7O[C@H]([C@H]([C@H]([C@@H]7O)O)O)C.[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[NH4+].[H]C([O-])=O', 'Salt containing 1 cation(s) and 1 "
               "anion(s)'), ('[H+].OC(CCC([O-])=O)C([O-])=O', 'Salt containing "
               "1 cation(s) and 1 anion(s)'), "
               "('[Se+4].CCN(CC)C([S-])=S.CCN(CC)C([S-])=S.CCN(CC)C([S-])=S.CCN(CC)C([S-])=S', "
               "'Salt containing 1 cation(s) and 4 anion(s)'), "
               "('O.O.[Cl-].[Cl-].C[N+](C)(C)CCOC(=O)CCC(=O)OCC[N+](C)(C)C', "
               "'Salt containing 1 cation(s) and 2 anion(s) and 2 neutral "
               "species'), ('O.O.O.O.O.O.O.[Co+2].S([O-])([O-])(=O)=O', 'Salt "
               "containing 1 cation(s) and 1 anion(s) and 7 neutral species'), "
               "('[Cl-].CCCCCCCCCC[N+](C)(C)CCCCCCCCCC', 'Salt containing 1 "
               "cation(s) and 1 anion(s)'), "
               "('[Al+3].N1=C2[N-]C(=NC3=NC(=NC4=NC(=NC=5[N-]C1=C6C5C(=C(C(=C6[H])[H])[H])[H])C=7C4=C(C(=C(C7[H])[H])[H])[H])C=8C3=C(C(=C(C8[H])[H])[H])[H])C=9C2=C(C(=C(C9[H])[H])[H])[H]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].O=C1N2C(C3=NC4=CC=CC=C4C=C3C2)=CC(=C1CO)[C@@](O)(C(=O)[O-])CC', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Li+].[Li+].[Li+].[O-]B([O-])[O-]', 'Salt containing 3 "
               "cation(s) and 1 anion(s)'), ('S=C1N([O-])C=CC=C1.[Na+]', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('C(CC(=O)[O-])C(C(=O)[O-])O.[Na+].[Na+]', 'Salt containing 2 "
               "cation(s) and 1 anion(s)'), ('[NH4+].NS([O-])(=O)=O', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('C1(C(NC([S-])=NC1=O)=O)(C(CCC)C)CC=C.[Na+]', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('[Cl-].CC(C[N+](C)(C)C)OC(C)=O', 'Salt containing 1 cation(s) "
               "and 1 anion(s)'), ('[I-].[S+](CC[C@H](N)C(O)=O)(C)C', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('[Ca+2].OC(C(CO)(C)C)C(=O)NCCC([O-])=O.OC(C(CO)(C)C)C(=O)NCCC([O-])=O', "
               "'Salt containing 1 cation(s) and 2 anion(s)'), "
               "('[Cl-].O1[C@@H](OC=2C(=[O+]C=3C(C2)=C(O)C=C(O)C3)C4=CC(OC)=C(O)C=C4)[C@H](O)[C@@H](O)[C@@H](O)C1', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Cl-].COc1ccc2cc3-c4cc5OCOc5cc4CC[n+]3cc2c1OC', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('C1CC1C2=NC3=CC=CC=C3C(=C2C=C[C@H](C[C@H](CC(=O)[O-])O)O)C4=CC=C(C=C4)F.C1CC1C2=NC3=CC=CC=C3C(=C2C=C[C@H](C[C@H](CC(=O)[O-])O)O)C4=CC=C(C=C4)F.[Ca+2]', "
               "'Salt containing 1 cation(s) and 2 anion(s)'), "
               "('[Na+].[O-]c1ccccc1', 'Salt containing 1 cation(s) and 1 "
               "anion(s)'), "
               "('S(O[C@H]1[C@H](O)C(C(=C(C1)C)\\\\C=C\\\\C(\\\\C)=C\\\\C=C\\\\C(\\\\C)=C\\\\C=C\\\\C=C(\\\\C=C\\\\C=C(\\\\C=C\\\\C=2C([C@@H](O)[C@H](O)C(=O)C2C)(C)C)/C)/C)(C)C)([O-])(=O)=O.[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[H][B-]([H])([H])[H].[H][N+]([H])([H])[H]', 'Salt containing "
               "1 cation(s) and 1 anion(s)'), ('[Na+].O=C(C(C)C)C([O-])=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('CC[NH+](CC)CC.CC[NH+](CC)CC.CN1c2cc3Oc4cc5=[N+](C)C(C)(C)C=C(CS([O-])(=O)=O)c5cc4=C(c3cc2C(CS([O-])(=O)=O)=CC1(C)C)c1c(Cl)c(SCC(=O)NCCCCCC(=O)ON2C(=O)CCC2=O)c(Cl)c(Cl)c1C([O-])=O', "
               "'Salt containing 2 cation(s) and 1 anion(s)'), "
               "('[Ca++].[I-].[I-]', 'Salt containing 1 cation(s) and 2 "
               "anion(s)'), ('[Na+].COc1cc(OC)nc(Sc2cccc(Cl)c2C([O-])=O)n1', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[NH4+].C1(=NC=C(C=C1C(=O)[O-])COC)C=2NC([C@@](N2)(C)C(C)C)=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('F[B-](F)(F)F.[O+]1=C(C(C)(C)C)C=C(C=C1C(C)(C)C)C2=CC=C(OC)C=C2', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[O-]C(=O)CS[Sb]1OC(=O)CS1', 'Salt containing 1 "
               "cation(s) and 1 anion(s)'), "
               "('[Ti-2]12(CC(C(C1)=O)C)(CC(C(C2)C)=O)(O)O.[NH4+].[NH4+]', "
               "'Salt containing 2 cation(s) and 1 anion(s)'), "
               "('S(OCC(\\\\C=C\\\\[C@H]([C@@]1([C@@]2([C@]([C@@]3(O)[C@@]([C@@]4([C@@]([C@H](O)C3)(C[C@@H](O[C@@H]5OC[C@@H](O)C(O)[C@@H]5O)CC4)[H])C)(CC2)[H])([C@@H](O)C1)[H])C)[H])C)C)([O-])(=O)=O.[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[H][C@@]12CC[C@]3(C)[C@@H](CCC4=CC(=O)OC4)\\\\C(CC[C@]3([H])[C@@]1(C)CCCC2(C)C)=C\\\\OS([O-])(=O)=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].C=1(S(NC=2C=CC=CC2C(=O)NCC([O-])=O)(=O)=O)C=CC(OC(C(C)(C)C)=O)=CC1.O.O.O.O', "
               "'Salt containing 1 cation(s) and 1 anion(s) and 4 neutral "
               "species'), "
               "('P(OC[C@@H](O)[C@@H](O)[C@H](O)C(=O)COP([O-])([O-])=O)(O)([O-])=O.[Na+].[Na+].[Na+]', "
               "'Salt containing 3 cation(s) and 1 anion(s)'), "
               "('S(O[C@H](CCCCCCCCC(=O)CCCCCC)[C@@](N)(CO)C(O)=O)([O-])(=O)=O.[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[NaH].[Na+].Cc1cc(C)c(cc1C)\\\\N=N\\\\c1c(O)c(cc2cc(ccc12)S([O-])(=O)=O)S([O-])(=O)=O', "
               "'Salt containing 1 cation(s) and 1 anion(s) and 1 neutral "
               "species'), "
               "('S([O-])(=O)(=O)C1=CC=C(\\\\N=N\\\\C=2C=3C(C=CC2O)=CC=CC3)C=C1.[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('S1C2N(C(C(=O)[O-])=C(C1)C=CC=3SC=NC3C)C(=O)C2NC(=O)C(=NOC)C=4N=C(N)SC4.[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Fe+3].[Fe+3].S(=O)([O-])(=O)[O-].S(=O)([O-])(=O)[O-].S(=O)([O-])(=O)[O-].O', "
               "'Salt containing 2 cation(s) and 3 anion(s) and 1 neutral "
               "species'), "
               "('C1=C(C=C2C(=C1)[N-]C(=N2)S(CC3=C(C(=C(C=N3)C)OC)C)=O)OC.[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[H+].OC(C(CC([O-])=O)C([O-])=O)C([O-])=O', 'Salt containing "
               "1 cation(s) and 1 anion(s)'), "
               "('O.[Cl-].[Cl-].CCO.[H][C@@]12[C@@H](C)c3cccc(O)c3C(=O)C1=C(O)[C@]1(O)C(=O)C(C(N)=O)=C(O)[C@@H]([NH+](C)C)[C@]1([H])[C@H]2O.[H][C@@]12[C@@H](C)c3cccc(O)c3C(=O)C1=C(O)[C@]1(O)C(=O)C(C(N)=O)=C(O)[C@@H]([NH+](C)C)[C@]1([H])[C@H]2O', "
               "'Salt containing 2 cation(s) and 2 anion(s) and 2 neutral "
               "species'), ('[NH4+].[NH4+].OP([O-])([O-])=O', 'Salt containing "
               "2 cation(s) and 1 anion(s)'), "
               "('[Br-].O(C1C2([C@](C3[C@@](C4([C@@](CC3)(C[C@H](OC(=O)C)C([N+]5(CCCCC5)C)C4)[H])C)(CC2)[H])(CC1[N+]6(CCCCC6)C)[H])C)C(=O)C', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].CO\\\\N=C(/C(=O)N[C@H]1[C@H]2SCC([C@@H]3CCCO3)=C(N2C1=O)C([O-])=O)c1csc(N)n1', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Fe+3].O=C1N[C@H](C(=O)NC(CN[C@H](C(=O)NCC(=O)NC(C(=O)NCC(OC[C@@H](C(NC(C(NCC(N[C@@H]1C(C)C)=O)=O)CCCN([O-])C(=O)C)=O)NC(=O)C/C=C\\\\CCCCCC)=O)CCCN([O-])C(=O)C)C)CCCN([O-])C(=O)C)CO', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[NH4+].[NH4+].[NH4+].N#C[Fe-3](C#N)(C#N)(C#N)(C#N)C#N', "
               "'Salt containing 3 cation(s) and 1 anion(s)'), "
               "('[Na+].[Na+].[H]O[H].[H]O[H].[H]O[H].[H]O[H].[H]O[H].[H]O[H].[H]O[H].[H]O[H].[H]O[H].[H]O[H].[O-]S([O-])(=O)=O', "
               "'Salt containing 2 cation(s) and 1 anion(s) and 10 neutral "
               "species'), ('[Cl-].O=C(CCC[NH+]1CCCC1)C2=C(OC)C=C(OC)C=C2OC', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[K+].[K+].[H]O[H].[H]O[H].[H]O[H].O=C1O[Sb-]23OC1C1O[Sb-]4(OC(C(O2)C(=O)O3)C(=O)O4)OC1=O', "
               "'Salt containing 2 cation(s) and 1 anion(s) and 3 neutral "
               "species'), "
               "('P(=O)(O)(CCC(N)C(=O)NC(C([O-])=NC(C(=O)O)C)C)C.[Na+]', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('[C@]12(N(C(=C(CS1)CSC3=NN=NN3C)C(=O)[O-])C([C@]2(NC(=O)[C@](NC(N4C(C(N(CC4)CC)=O)=O)=O)([C@@H](O)C)[H])OC)=O)[H].[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[NH3+]CCO.CC(Cc1cc(C)cc(=O)n1[O-])CC(C)(C)C', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('[O-]C(=O)[C@@H]([C@H]([C@H](O)[C@@H](CO)O)O)O.[K+]', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('C(CN(C(/C=C(/CCO)\\\\C)=O)[O-])C[C@@H](C(=O)OCC/C(/C)=C/C(=O)N([O-])CCC[C@@]1(NC([C@](CCCN(C(/C=C(/CCO)\\\\C)=O)[O-])(NC1=O)[H])=O)[H])N.[Fe+3]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].OC(=O)c1nc(c[nH]1)C(=O)N[C@@H](C(=O)N[C@H]1[C@H]2SCC(C[n+]3ccc(CCS([O-])(=O)=O)cc3)=C(N2C1=O)C([O-])=O)c1ccccc1', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('S(O[C@@H]1[C@@]2([C@@]([C@]([C@H]1O)([C@@H](CCC[C@@H](CO)C)C)[H])(CC[C@]3([C@@]2(O)C[C@@H](O)C=4[C@@]3(CC[C@H](O[C@@H]5OC[C@@H](O)C(O)[C@@H]5OC)C4)C)[H])C)[H])([O-])(=O)=O.[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].[Na+].[O-]C(=O)[C@H]1N(\\\\[N+](\\\\[O-])=N\\\\[O-])CCC1', "
               "'Salt containing 2 cation(s) and 1 anion(s)'), "
               "('[Cl-].CC[n+]1c(-c2ccccc2)c2cc(N\\\\N=N\\\\c3cccc(c3)C(N)=N)ccc2c2ccc(N)cc12', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[H+].O[C@@H]([C@@H](O)[C@H](O)C([O-])=O)[C@H](O)C([O-])=O', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), ('[Cl-].[Au+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Ca++].[O-]C=O.[O-]C=O', 'Salt containing 1 cation(s) and 2 "
               "anion(s)'), "
               "('[H+].[Na+].CC1CC(C)(C)Nc2c1cc1c(Oc3c(c4=NC(C)(C)CC(C)c4cc3=C1c1c(Cl)c(SCC(=O)NCCCCCC(=O)ON3C(=O)CCC3=O)c(Cl)c(Cl)c1C(O)=O)S([O-])(=O)=O)c2S([O-])(=O)=O', "
               "'Salt containing 2 cation(s) and 1 anion(s)'), "
               "('[Zn+2].S(O[H])(=O)(=O)C1=C(C2=C(C=3[N-]C2=NC4=NC(=NC=5[N-]C(N=C6N=C(N3)C7=C6C(=C(C(S(O[H])(=O)=O)=C7[H])[H])[H])=C8C5C(=C(C(S(O[H])(=O)=O)=C8[H])[H])[H])C9=C4C(=C(C(S(O[H])(=O)=O)=C9[H])[H])[H])C(=C1[H])[H])[H]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[Na+].Cc1conc1[N-]S(=O)(=O)c1ccc(N)cc1', 'Salt containing 1 "
               "cation(s) and 1 anion(s)'), "
               "('[O-]Cl(=O)(=O)=O.CN(C)c1ccc2C=c3cc4CCC[N+](CCCC(=O)NCCN5C(=O)C=CC5=O)=c4cc3C(C)(C)c2c1', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('O.O.O.[Ca++].CC(C)c1c(C(=O)Nc2ccccc2)c(-c2ccccc2)c(-c2ccc(F)cc2)n1CC[C@@H](O)C[C@@H](O)CC([O-])=O.CC(C)c1c(C(=O)Nc2ccccc2)c(-c2ccccc2)c(-c2ccc(F)cc2)n1CC[C@@H](O)C[C@@H](O)CC([O-])=O', "
               "'Salt containing 1 cation(s) and 2 anion(s) and 3 neutral "
               "species'), "
               "('[Cl-].O1[C@@H](OC=2C(=[O+]C=3C(C2)=C(O)C=C(O)C3)C4=CC(O)=C(O)C(O)=C4)[C@H](O)[C@@H](O)[C@@H](O)C1', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('P(=O)([O-])([O-])[O-].[Mg+2].[NH4+].O', 'Salt containing 2 "
               "cation(s) and 1 anion(s) and 1 neutral species'), "
               "('[Cl-].O(CC[N+](CC1=CC=CC=C1)(C)C)CCOC2=C(C=C(C(CC(C)(C)C)(C)C)C=C2)C', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('OC[C@H]([C@@H](O)[C@H](O)[C@@H](O)C([O-])=O)O.[Fe+3].OC[C@H]([C@@H](O)[C@H](O)[C@@H](O)C([O-])=O)O.OC[C@H]([C@@H](O)[C@H](O)[C@@H](O)C([O-])=O)O', "
               "'Salt containing 1 cation(s) and 3 anion(s)'), "
               "('[Bi+3].OC(CC([O-])=O)(CC([O-])=O)C([O-])=O', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('[Al+3].CCCCCCCCCCCCCCCCCC([O-])=O.CCCCCCCCCCCCCCCCCC([O-])=O.CCCCCCCCCCCCCCCCCC([O-])=O', "
               "'Salt containing 1 cation(s) and 3 anion(s)'), "
               "('[V](=O)(=O)[O-].[Na+].O', 'Salt containing 1 cation(s) and 1 "
               "anion(s) and 1 neutral species'), "
               "('[Mg++].OC[C@H](O)[C@H]1OC(=O)C(OP([O-])([O-])=O)=C1O', 'Salt "
               "containing 1 cation(s) and 1 anion(s)'), "
               "('S(OP(OC[C@H]1O[C@@H](N2C3=NC=NC(N)=C3N=C2)[C@H](O)[C@@H]1O)(O)=O)([O-])(=O)=O.[Na+]', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('CC[NH+](CC)CC.CCCCCCCCCCCCCCCC(=O)OCC(COP([O-])(=O)OCCNS(=O)(=O)c1ccc(c(c1)S([O-])(=O)=O)C1=c2cc3CCC[N+]4=c3c(CCC4)c2Oc2c3CCCN4CCCc(cc12)c34)OC(=O)CCCCCCCCCCCCCCC', "
               "'Salt containing 1 cation(s) and 1 anion(s)'), "
               "('[K+].[O-]C(=O)c1nc(=O)[nH]c(=O)[nH]1', 'Salt containing 1 "
               "cation(s) and 1 anion(s)'), ('O.O.O.O.O.O.[Cl-].[Cl-].[Co++]', "
               "'Salt containing 1 cation(s) and 2 anion(s) and 6 neutral "
               "species'), ('[Br-].C1CCCC1C(C(OCC[N+](CC)(CC)CC)=O)C2CCCC2', "
               "'Salt containing 1 cation(s) and 1 anion(s)')]\n"
               'False negatives: '
               "[('CC(=O)NCC(O)=O.CC(=O)NCC(O)=O.NC(=N)c1ccc(NN=Nc2ccc(cc2)C(N)=N)cc1', "
               "'No charged species present'), "
               "('Cl.Cc1c(oc2c(cccc2c1=O)C(=O)OCCN1CCCCC1)-c1ccccc1', 'No "
               "charged species present'), "
               "('CS(O)(=O)=O.C1CCC(CC1)c1ccc2n3CCN[C@@H]4CCCc(c34)c2c1', 'No "
               "charged species present'), "
               "('CS(O)(=O)=O.N[C@H]1[C@@H]2CN(C[C@H]12)c1nc2n(cc(C(O)=O)c(=O)c2cc1F)-c1ccc(F)cc1F', "
               "'No charged species present'), "
               "('Cl.Cc1ncc(CNC(=O)N(CCCl)N=O)c(N)n1', 'No charged species "
               "present'), ('NCCNCCN.C1C(CCl)O1.Cl', 'No charged species "
               "present'), "
               "('CC(O)=O.CCNC(=O)[C@@H]1CCCN1C(=O)[C@H](CCCNC(N)=N)NC(=O)[C@H](CC(C)C)NC(=O)[C@@H](CC(C)C)NC(=O)[C@H](Cc1ccc(O)cc1)NC(=O)[C@H](CO)NC(=O)[C@H](Cc1c[nH]c2ccccc12)NC(=O)[C@H](Cc1c[nH]cn1)NC(=O)[C@@H]1CCC(=O)N1', "
               "'No charged species present'), ('[Li]C#CC', 'Single molecular "
               "entity - not a salt'), ('Cl.COc1ccc(OC)c(c1)C(O)CNC(=O)CN', "
               "'No charged species present'), "
               "('O[N+]([O-])=O.Clc1ccc([C@H](Cn2ccnc2)OCc2ccc(Sc3ccccc3)cc2)c(Cl)c1', "
               "'No charged species present'), "
               "('Cl.C1CCC(CC1)c1ccc2n3CCN[C@H]4CCCc(c34)c2c1', 'No charged "
               "species present'), "
               "('OC(=O)\\\\C=C/C(O)=O.COc1ccc(cn1)-c1ccc2ncc3n(C)c(=O)n(-c4ccc(N5CCNCC5)c(c4)C(F)(F)F)c3c2c1', "
               "'No charged species present'), "
               "('Cl.N(c1ccccc1)c1ccc(cc1)C(=C1C=CC(C=C1)=Nc1ccccc1)c1ccc(Nc2ccccc2)cc1', "
               "'No charged species present'), "
               "('Cl.COC1=CC2=C(CCCC2C(=O)N(CC2=CC=C(C=C2)N(C)C)C2=CC=C(C=C2)C(C)C)C=C1', "
               "'No charged species present'), "
               "('C1[C@@]2(N3CC[C@@]42[C@]5(N(C=6C4=CC=CC6)C(C[C@]7([C@@]5([C@@]1(C(=CCO7)C3)[H])[H])[H])=O)[H])[H].Cl', "
               "'No charged species present'), "
               "('Cl.C[C@@H](NCCCc1cccc(c1)C(F)(F)F)c1cccc2ccccc12', 'No "
               "charged species present'), "
               "('OC(=O)CC(O)(CC(O)=O)C(O)=O.C[C@@H]1CCN(C[C@@H]1N(C)c1ncnc2[nH]ccc12)C(=O)CC#N', "
               "'No charged species present'), "
               "('CCCCCCCCCCCCOS(O)(=O)=O.CC[C@H]1OC(=O)[C@H](C)[C@@H](O[C@H]2C[C@@](C)(OC)[C@@H](O)[C@H](C)O2)[C@H](C)[C@@H](O[C@@H]2O[C@H](C)C[C@@H]([C@H]2OC(=O)CC)N(C)C)[C@](C)(O)C[C@@H](C)C(=O)[C@H](C)[C@@H](O)[C@]1(C)O', "
               "'No charged species present'), "
               "('Cc1ccc(cc1)S(O)(=O)=O.N[C@@H]1CCN(C1)c1nc2n(cc(C(O)=O)c(=O)c2cc1F)-c1ccc(F)cc1F', "
               "'No charged species present'), "
               "('O[N+]([O-])=O.Clc1ccc([C@H](Cn2ccnc2)OCc2c(Cl)cccc2Cl)c(Cl)c1', "
               "'No charged species present'), "
               "('Cl.CNCC[C@H](Oc1ccc(cc1)C(F)(F)F)c1ccccc1', 'No charged "
               "species present'), "
               "('Br.CN1CCc2c(cc(O)c(O)c2Cl)C(C1)c1cccc(C)c1', 'No charged "
               "species present'), "
               "('Cl.Cl.C\\\\C(\\\\C=N\\\\NC(N)=N)=N/NC(N)=N', 'No charged "
               "species present'), "
               "('Cl.CN1CCCC(CC1)n1nc(Cc2ccc(Cl)cc2)c2ccccc2c1=O', 'No charged "
               "species present'), ('Cl.COC1=C(OC)C=C2CNCCC2=C1', 'No charged "
               "species present'), "
               "('Cl.CN(C)C(=O)Oc1cc(OC(=O)N(C)C)cc(c1)C(O)CNC(C)(C)C', 'No "
               "charged species present'), "
               "('Cl.CC[C@@]1(O)C[C@H](O[C@H]2C[C@@H]([C@H](O)[C@H](C)O2)N(C)C)c2c(O)c3C(=O)c4c(O)cccc4C(=O)c3cc2[C@H]1C(=O)OC', "
               "'No charged species present'), "
               "('Cl.CN(C)C(=O)Oc1cc(OC(=O)N(C)C)cc(c1)[C@H](O)CNC(C)(C)C', "
               "'No charged species present'), "
               "('Br.CC(Cc1ccc(O)cc1)NCC(O)c1cc(O)cc(O)c1', 'No charged "
               "species present'), "
               "('Cl.O=C1N(C[C@H]2CCCc3cccc1c23)[C@@H]1CN2CCC1CC2', 'No "
               "charged species present'), "
               "('Cl.CN1CCN(CC1)C(c1ccccc1)c1ccccc1', 'No charged species "
               "present'), "
               "('OS(=O)(=O)c1ccccc1.CCOC(=O)C1=C(COCCN)NC(C)=C(C1c1ccccc1Cl)C(=O)OC', "
               "'No charged species present'), "
               "('Cl.COc1ccc(CCN(C)CCC[C@](C#N)(C(C)C)c2ccc(OC)c(OC)c2)cc1OC', "
               "'No charged species present'), "
               "('Cl.Clc1ccc2N=C3NC(=O)CN3Cc2c1Cl', 'No charged species "
               "present'), ('C1=2C(C=CC=C1)=NC(=CC2)/C=C/C=3C(=CC=CC3)O.Cl', "
               "'No charged species present'), "
               "('Cl.CCN1CCC(C1C)=C1C2=CC=CC=C2CCC2=CC=CC=C12', 'No charged "
               "species present'), "
               "('Br.COc1ccccc1N1CCN(CCCCN2C(=O)c3ccccc3C2=O)CC1', 'No charged "
               "species present'), ('Br.CC1NCCC2=CC(O)=C(O)C=C12', 'No charged "
               "species present'), "
               "('Cl.OC(=O)c1cn(C2CC2)c2cc(N3CCNCC3)c(F)cc2c1=O', 'No charged "
               "species present'), ('Cl.CN1CCC(CC1)=C1c2ccccc2C=Cc2ccccc12', "
               "'No charged species present'), "
               "('Cl.C1CCC(CC1)c1ccc2n3CCN[C@@H]4CCCc(c34)c2c1', 'No charged "
               "species present'), ('Cl.NCCCC[C@H](N)C(O)=O', 'No charged "
               "species present'), "
               "('NC(CO)(CO)CO.NC(CO)(CO)CO.OC(=O)\\\\C=C/C(O)=O', 'No charged "
               "species present'), "
               "('Cl.CCCCCC[C@@H]([C@@H](C)O)n1cnc2c(N)ncnc12', 'No charged "
               "species present'), "
               "('C=1C(=CC=C(C1)N)C(=C2C=CC(=N)C=C2)C=3C=CC(=CC3)N.Cl', 'No "
               "charged species present'), "
               "('OS(O)(=O)=O.NC[C@H]1O[C@H](O[C@@H]2[C@@H](N)C[C@@H](N)[C@H](O[C@H]3O[C@H](CO)[C@@H](O)[C@H](N)[C@H]3O)[C@H]2O)[C@H](N)[C@@H](O)[C@@H]1O', "
               "'No charged species present'), ('Cl.NOCC(O)=O.NOCC(O)=O', 'No "
               "charged species present'), "
               "('CC(O)=O.CC(C)C[C@H](NC(=O)[C@@H](CC1=CC2=C(C=CC=C2)C=C1)NC(=O)[C@H](CC1=CC=C(O)C=C1)NC(=O)[C@H](CO)NC(=O)[C@H](CC1=CNC2=C1C=CC=C2)NC(=O)[C@H](CC1=CNC=N1)NC(=O)[C@@H]1CCC(=O)N1)C(=O)N[C@@H](CCCNC(N)=N)C(=O)N1CCC[C@H]1C(=O)NCC(N)=O', "
               "'No charged species present'), "
               "('O[N+]([O-])=O.Clc1ccc([C@@H](Cn2ccnc2)OCc2csc3c(Cl)cccc23)c(Cl)c1', "
               "'No charged species present'), "
               "('C=1C(=CC(=C(C1)N)C)C(=C2C=CC(=N)C=C2)C=3C=C(C(=CC3)N)C.Cl', "
               "'No charged species present'), "
               "('C(=C\\\\SC)(\\\\C1=C(OCCCCC)C=CC=C1)/N2C=CN=C2.Cl', 'No "
               "charged species present'), "
               "('CS(O)(=O)=O.CCCCCCOC(=O)\\\\N=C(\\\\N)c1ccc(NCc2nc3cc(ccc3n2C)C(=O)N(CCC(=O)OCC)c2ccccn2)cc1', "
               "'No charged species present'), ('O.O.O.O.Cl[Mn]Cl', 'No "
               "charged species present')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 143,
    'num_false_positives': 100,
    'num_true_negatives': 14008,
    'num_false_negatives': 22,
    'num_negatives': None,
    'precision': 0.588477366255144,
    'recall': 0.8666666666666667,
    'f1': 0.7009803921568626,
    'accuracy': 0.9914523926294402}


"""
Classifies: CHEBI:24675 hydroxybenzoate
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_hydroxybenzoate(smiles: str):
    """
    Determines if a molecule is a hydroxybenzoate (benzoate derivative with carboxylate and at least one hydroxy group).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a hydroxybenzoate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for carboxylate group attached to aromatic ring
    carboxylate_pattern = Chem.MolFromSmarts('c-C(=O)[O-]')
    if not mol.HasSubstructMatch(carboxylate_pattern):
        return False, "No carboxylate group directly attached to aromatic ring"

    # Count carboxylate groups attached to aromatic rings
    carboxylate_matches = mol.GetSubstructMatches(carboxylate_pattern)
    if len(carboxylate_matches) != 1:
        return False, "Must have exactly one carboxylate group"

    # Get the aromatic carbon attached to carboxylate
    carboxylate_carbon = carboxylate_matches[0][0]

    # Find the aromatic ring containing the carboxylate
    rings = mol.GetRingInfo()
    carboxylate_ring = None
    for ring in rings.AtomRings():
        if carboxylate_carbon in ring:
            ring_atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetIsAromatic() for atom in ring_atoms):
                if all(mol.GetAtomWithIdx(i).GetSymbol() == 'C' for i in ring):
                    carboxylate_ring = set(ring)
                    break

    if not carboxylate_ring:
        return False, "Carboxylate not attached to benzene ring"

    # Check for hydroxyl groups
    hydroxyl_pattern = Chem.MolFromSmarts('[cH0]-[OH]')
    if not mol.HasSubstructMatch(hydroxyl_pattern):
        return False, "No hydroxyl group found"

    # Count hydroxyl groups on the aromatic system
    hydroxyl_matches = mol.GetSubstructMatches(hydroxyl_pattern)
    hydroxyl_count = 0
    hydroxyl_on_ring = False

    for match in hydroxyl_matches:
        aromatic_carbon = match[0]
        if aromatic_carbon in carboxylate_ring:
            hydroxyl_on_ring = True
            hydroxyl_count += 1

    if not hydroxyl_on_ring:
        return False, "No hydroxyl group on same ring as carboxylate"

    return True, f"Hydroxybenzoate with {hydroxyl_count} hydroxyl group(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24675',
                          'name': 'hydroxybenzoate',
                          'definition': 'Any benzoate derivative carrying a '
                                        'single carboxylate group and at least '
                                        'one hydroxy substituent.',
                          'parents': ['CHEBI:22718', 'CHEBI:36059']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.13333333333333333 is too low.\n'
               "True positives: [('C=1(C=C(C(=C(C1)Br)O)O)C([O-])=O', "
               "'Hydroxybenzoate with 2 hydroxyl group(s)'), "
               "('COC1=C(O)C=CC(\\\\C=C\\\\C(=O)NC2=CC=C(O)C=C2C([O-])=O)=C1', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('Nc1c(O)cccc1C([O-])=O', 'Hydroxybenzoate with 1 hydroxyl "
               "group(s)'), "
               "('COc1cc(cc(C\\\\C=C(/C)CC\\\\C=C(/C)CC\\\\C=C(/C)CC\\\\C=C(/C)CC\\\\C=C(/C)CC\\\\C=C(/C)CC\\\\C=C(/C)CCC=C(C)C)c1O)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C=1C(=CC(=C(O)C1OC)OC)C([O-])=O', 'Hydroxybenzoate with 1 "
               "hydroxyl group(s)'), ('[O-]C(=O)C1=CC=CC(=C1O)Cl', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)')]\n"
               'False positives: '
               "[('C=1(C(=CC(O)=CC1)C([O-])=O)NC(/C=C/C2=CC=C(O)C=C2)=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('O.O.[Na+].[Na+].Oc1ccc(cc1C([O-])=O)\\\\N=N\\\\c1ccc(cc1)C(=O)NCCC([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C1(=C(C(=CC(=C1)O)O)C([O-])=O)/C=C/CCCC(=O)CCC[C@@H](O)C', "
               "'Hydroxybenzoate with 2 hydroxyl group(s)'), "
               "('Cc1cccc2c(cc(O)cc12)C([O-])=O', 'Hydroxybenzoate with 1 "
               "hydroxyl group(s)'), "
               "('C=1(C2=C(C(=CC1C(=O)[O-])Cl)C[C@H](OC2=O)C)O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C=1(C(=C2C(C3=C(C=C(O)C(C([O-])=O)=C3C)C(C2=C(O)C1[O-])=O)=O)O)[C@@H]4O[C@@H]([C@H](C([C@H]4O)=O)O)CO', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C=1C(=C2C(C3=C(C=C(O)C(C([O-])=O)=C3C)C(C2=CC1O)=O)=O)O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('[H][C@@]12CCC(C)=C[C@@]1([H])c1c(O)c(C([O-])=O)c(CCCCC)cc1OC2(C)C', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\Cc1cc(ccc1O)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\Cc1cc(ccc1O)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('[Ca+2].O1[C@]([C@H](O)[C@@H](O)[C@H](O)[C@H]1CO)(C2=C3C(=C([O-])C=C2)C(=O)C4=C(C3=O)C=C(C=C4O)C([O-])=O)[H]', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('Oc1ccc(C([O-])=O)c(O)n1', 'Hydroxybenzoate with 2 hydroxyl "
               "group(s)'), "
               "('C(C\\\\C(=C\\\\CC\\\\C(=C\\\\CC\\\\C(=C\\\\CC\\\\C(=C\\\\CC1=CC(=CC(=C1N)O)C([O-])=O)\\\\C)\\\\C)\\\\C)\\\\C)/C=C(/CCC=C(C)C)\\\\C', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('[Na+].[H][C@@]1(O[C@@](CC)(C[C@@H]1C)[C@@]1([H])CC[C@](O)(CC)[C@H](C)O1)[C@@H](CC)C(=O)[C@@H](C)[C@@H](O)[C@H](C)CCC1=C(C([O-])=O)C(O)=C(C)C=C1', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('[Na+].[Na+].[Na+].CC1=C\\\\C(C=C(C([O-])=O)C1=O)=C(\\\\c1cc(C)c(O)c(c1)C([O-])=O)c1ccccc1S([O-])(=O)=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('OC1C=Cc2c(O)cc(nc2C1O)C([O-])=O', 'Hydroxybenzoate with 1 "
               "hydroxyl group(s)'), "
               "('OC(=O)C(=O)CCc1c(O)cc([nH]c1=O)C([O-])=O', 'Hydroxybenzoate "
               "with 1 hydroxyl group(s)'), "
               "('Oc1cc([nH]c(=O)c1CCC=O)C([O-])=O', 'Hydroxybenzoate with 1 "
               "hydroxyl group(s)'), ('C=1C=C(C=C(C1O)OC)C([O-])=O.[Na+]', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C(C=1C=CC(=CC1)O)([O-])=O.[Na+]', 'Hydroxybenzoate with 1 "
               "hydroxyl group(s)'), ('Cc1ncc(cc1O)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('Oc1ccc([nH]1)C([O-])=O', 'Hydroxybenzoate with 1 hydroxyl "
               "group(s)'), ('Oc1cc(O)c(C([O-])=O)c(OC(=O)c2ccc(O)c(O)c2)c1', "
               "'Hydroxybenzoate with 2 hydroxyl group(s)'), "
               "('Cc1cccc2c(C([O-])=O)c(O)ccc12', 'Hydroxybenzoate with 1 "
               "hydroxyl group(s)'), "
               "('Cc1c(C([O-])=O)c(O)cc2cc3Cc4cc(O)cc(O)c4C(=O)c3c(O)c12', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C1(C(=O)[O-])=CC(=C(C(=C1)O)O)O.[Na+]', 'Hydroxybenzoate "
               "with 3 hydroxyl group(s)'), ('Oc1c(ccc2ccccc12)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('[H]C(=CC(=O)C([O-])=O)c1c(O)cc(nc1O)C([O-])=O', "
               "'Hydroxybenzoate with 2 hydroxyl group(s)'), "
               "('C1(=C(C=CC(=C1O)Cl)Cl)C([O-])=O', 'Hydroxybenzoate with 1 "
               "hydroxyl group(s)'), ('[Na+].Nc1ccc(C([O-])=O)c(O)c1', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C=1(C(=C2C(C3=C(C=C(O)C(C([O-])=O)=C3C)C(C2=CC1[O-])=O)=O)O)[C@@H]4O[C@@H]([C@H]([C@@H]([C@H]4O)O)O)CO', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C=1(C([O-])=O)C=CC(=C(C1O)N)OC', 'Hydroxybenzoate with 1 "
               "hydroxyl group(s)'), ('Oc1cccc2c(O)cc(nc12)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('Oc1cc(C([O-])=O)c(O)c2ccccc12', 'Hydroxybenzoate with 2 "
               "hydroxyl group(s)'), ('C1(=CC=C(C=C1NC(C)=O)C([O-])=O)O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('Cc1c(C([O-])=O)c(O)cc2cc3C(=O)c4cc([O-])cc(O)c4C(=O)c3c(O)c12', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('[NH3+]C(CC(=O)c1cccc2oc3cc(=O)c4nc(cc(O)c4c3nc12)C([O-])=O)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C1(=C(C=CC=C1)O)C(=O)[O-].C[N+](CCO)(C)C', 'Hydroxybenzoate "
               "with 1 hydroxyl group(s)'), "
               "('C1(=CN=C(C(=C1C(=O)[O-])O)C)C=O', 'Hydroxybenzoate with 1 "
               "hydroxyl group(s)'), "
               "('[NH3+]C(CC(=O)c1cccc2Oc3cc(O)c4nc(cc(O)c4c3Nc12)C([O-])=O)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('COc1cc(C)c2ccc(O)c(C([O-])=O)c2c1', 'Hydroxybenzoate with 1 "
               "hydroxyl group(s)'), ('[Na+].C=1C(=CC(=C(O)C1OC)OC)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('Cc1cc(O)cc2c(C([O-])=O)c(O)ccc12', 'Hydroxybenzoate with 1 "
               "hydroxyl group(s)'), ('Cc1ncc(C([O-])=O)c(CO)c1O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('[Na+].OC1=C(C=C(/N=N/C2=CC=C(N)C=C2)C=C1)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('COc1cc(cc(O)c1O)C([O-])=O', 'Hydroxybenzoate with 2 hydroxyl "
               "group(s)'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\Cc1cc(ccc1O)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('Oc1ccc(cn1)C([O-])=O', 'Hydroxybenzoate with 1 hydroxyl "
               "group(s)'), ('Oc1cc(cc(O)c(=O)c1)C([O-])=O', 'Hydroxybenzoate "
               "with 2 hydroxyl group(s)'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\Cc1cc(ccc1O)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C=1C(=C2C(C3=C(C=C(O)C(C([O-])=O)=C3C)C(C2=C(O)C1O)=O)=O)O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('Oc1ccccc1C([O-])=O.C[NH+](C)CCOC(c1ccccc1)c1ccccc1', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('[Na+].Oc1ccccc1C([O-])=O', 'Hydroxybenzoate with 1 hydroxyl "
               "group(s)'), ('Oc1ccc2c(O)cc(nc2c1O)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('Oc1ccc(cc1C([O-])=O)\\\\N=N\\\\c1ccc(cc1)C(=O)NCCC([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('CC1=CC(=C(C)C(=C1C(=O)OC=2C=C(C)C(=C(C2C)O)C(=O)[O-])O)O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('COc1cc(O)cc(C([O-])=O)c1C(=O)c1c(O)cc(C)cc1O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('OC(=O)C(=O)\\\\C=C\\\\c1c(O)cc([nH]c1=O)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('Cc1c(C([O-])=O)c(O)cc2cc3C(=O)c4cc(O)cc(O)c4C(=O)c3c(O)c12', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C1=2C(=C(C=C3C(N(C(C(=CC=C1)C23)=O)C=4C=CC(=C(C4)C(=O)[O-])O)=O)S([O-])(=O)=O)N.[Na+].[Na+]', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C1(=C(C2=C(C=CC=C2)C(=C1CC=C(C)C)O)O)C(=O)[O-]', "
               "'Hydroxybenzoate with 2 hydroxyl group(s)'), "
               "('Cc1ncc(CO)c(C([O-])=O)c1O', 'Hydroxybenzoate with 1 hydroxyl "
               "group(s)'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\Cc1cc(ccc1O)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('[Na+].OC1=C(C=C(O)C=C1)C([O-])=O', 'Hydroxybenzoate with 2 "
               "hydroxyl group(s)'), "
               "('CC1=CC(=C(C=O)C(=C1C(=O)OC=2C=C(C)C(=C(C2C)O)C(=O)[O-])O)O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('COc1cc(O)cc(C([O-])=O)c1C(=O)c1c(O)cc(C)cc1[O-]', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('CCCCCc1cc(O)cc(O)c1C([O-])=O', 'Hydroxybenzoate with 2 "
               "hydroxyl group(s)'), ('C=1(C=CC(=CC1C(=O)[O-])O)N', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C(C=1C(O)=C2C(CC(C)OC2=O)=CC1)([O-])=O', 'Hydroxybenzoate "
               "with 1 hydroxyl group(s)'), "
               "('[Na+].O1C(CC(C1C(CC)C(=O)C(C(O)C(CCC2=C(C(O)=C(C=C2)C)C([O-])=O)C)C)C)(C3OC(C(O)(CC3)CC)C)CC', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('Cc1cc(O)cc(O)c1C(=O)Oc1cc(C)c(C([O-])=O)c(O)c1', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('C=1(C(=C2C(C3=C(C=C(O)C(C([O-])=O)=C3C)C(C2=C(O)C1[O-])=O)=O)O)[C@@H]4O[C@@H]([C@H]([C@@H]([C@H]4O)O)O)CO', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('Oc1cc(nc2ccccc12)C([O-])=O', 'Hydroxybenzoate with 1 "
               "hydroxyl group(s)'), "
               "('CC1=CC(=C(CO)C(=C1C(=O)OC=2C=C(C)C(=C(C2C)O)C(=O)[O-])O)O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('CCCCCC1=CC2=C(C=CC(C)(CCC=C(C)C)O2)C(O)=C1C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('[H][C@@]1(O[C@@](CC)(C[C@@H]1C)[C@@]1([H])CC[C@](O)(CC)[C@H](C)O1)[C@@H](CC)C(=O)[C@@H](C)[C@@H](O)[C@H](C)CCC1=C(C([O-])=O)C(O)=C(C)C=C1', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)'), "
               "('OC(=O)CCc1c(O)cc([nH]c1=O)C([O-])=O', 'Hydroxybenzoate with "
               "1 hydroxyl group(s)'), ('Oc1cc(cc([O-])c(=O)c1)C([O-])=O', "
               "'Hydroxybenzoate with 1 hydroxyl group(s)')]\n"
               'False negatives: []',
    'attempt': 4,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 6,
    'num_false_positives': 60,
    'num_true_negatives': 183809,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.09090909090909091,
    'recall': 1.0,
    'f1': 0.16666666666666669,
    'accuracy': 0.9996736913664174}


"""
Classifies: CHEBI:13209 long-chain fatty acid ethyl ester
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_long_chain_fatty_acid_ethyl_ester(smiles: str):
    """
    Determines if a molecule is a long-chain fatty acid ethyl ester.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a long-chain fatty acid ethyl ester, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for presence of ethyl ester group (C(=O)OCC)
    ethyl_ester_pattern = Chem.MolFromSmarts('C(=O)OCC')
    if not mol.HasSubstructMatch(ethyl_ester_pattern):
        return False, "No ethyl ester group found"
    
    # Get carbon chain length
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count < 12:  # Generally long chain fatty acids have 12+ carbons
        return False, "Carbon chain too short for long-chain fatty acid"

    # Check for linear chain structure
    # Count number of carbons with more than 2 carbon neighbors (branching points)
    branching_points = 0
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            carbon_neighbors = sum(1 for neighbor in atom.GetNeighbors() 
                                if neighbor.GetSymbol() == 'C')
            if carbon_neighbors > 2:
                branching_points += 1
    
    if branching_points > 0:
        return False, "Structure contains branching points"

    # Count double bonds using SMARTS pattern
    double_bond_pattern = Chem.MolFromSmarts('C=C')
    double_bonds = len(mol.GetSubstructMatches(double_bond_pattern))
    
    # Classify based on saturation
    if double_bonds == 0:
        return True, f"Saturated long-chain fatty acid ethyl ester with {carbon_count} carbons"
    else:
        return True, f"Unsaturated long-chain fatty acid ethyl ester with {carbon_count} carbons and {double_bonds} double bonds"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:13209',
                          'name': 'long-chain fatty acid ethyl ester',
                          'definition': 'A fatty acid ethyl ester resulting '
                                        'from the formal condensation of the '
                                        'carboxy group of a long-chain fatty '
                                        'acid with the hydroxy group of '
                                        'ethanol.',
                          'parents': ['CHEBI:78206']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: module 'rdkit.Chem.rdMolDescriptors' has no "
               "attribute 'CalcNumAliphaticDoubleBonds'",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 100,
    'num_true_negatives': 1762,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0196078431372549,
    'recall': 1.0,
    'f1': 0.038461538461538464,
    'accuracy': 0.9463519313304721}


"""
Classifies: CHEBI:17984 acyl-CoA
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_acyl_CoA(smiles: str):
    """
    Determines if a molecule is an acyl-CoA (thioester formed between coenzyme A and a carboxylic acid).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an acyl-CoA, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for thioester group (R-C(=O)-S-R)
    thioester_pattern = Chem.MolFromSmarts('[CX3](=O)[SX2]')
    if not mol.HasSubstructMatch(thioester_pattern):
        return False, "No thioester group found"

    # Check for key parts of CoA structure
    # Adenine base
    adenine_pattern = Chem.MolFromSmarts('c1nc(c2c(n1)n(cn2)[C@H]3O[C@H](COP)C[C@@H]3O)N')
    
    # Pantetheine part with thiol
    pantetheine_pattern = Chem.MolFromSmarts('NCCSC(=O)')
    
    # Phosphate groups
    phosphate_pattern = Chem.MolFromSmarts('OP(O)(O)=O')

    if not (mol.HasSubstructMatch(adenine_pattern) and 
            mol.HasSubstructMatch(pantetheine_pattern) and
            mol.HasSubstructMatch(phosphate_pattern)):
        return False, "Missing key components of Coenzyme A structure"

    # Get the acyl part by finding atoms connected to thioester
    thioester_matches = mol.GetSubstructMatches(thioester_pattern)
    if thioester_matches:
        carbonyl_idx = thioester_matches[0][0]
        carbonyl_atom = mol.GetAtomWithIdx(carbonyl_idx)
        
        # Get neighbors excluding O and S atoms to identify R group
        r_group_atoms = [n for n in carbonyl_atom.GetNeighbors() 
                        if n.GetSymbol() not in ['O','S']]
        
        if r_group_atoms:
            r_group_type = "substituted"
            return True, f"Acyl-CoA with {r_group_type} acyl group"
    
    return True, "Acyl-CoA"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17984',
                          'name': 'acyl-CoA',
                          'definition': 'A thioester that results from the '
                                        'formal condensation of the thiol '
                                        'group of coenzyme A with the carboxy '
                                        'group of any carboxylic acid.',
                          'parents': [   'CHEBI:140325',
                                         'CHEBI:20706',
                                         'CHEBI:231540',
                                         'CHEBI:51277']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 79,
    'num_false_positives': 100,
    'num_true_negatives': 15907,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.441340782122905,
    'recall': 1.0,
    'f1': 0.6124031007751938,
    'accuracy': 0.9937834141489494}


"""
Classifies: CHEBI:23849 diterpenoid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_diterpenoid(smiles: str):
    """
    Determines if a molecule is a diterpenoid based on its SMILES string.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a diterpenoid, False otherwise
        str: Reason for classification
    """
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check carbon count - diterpenoids have C20 skeleton base
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count < 18 or carbon_count > 22: # Allow some variation from C20
        return False, f"Carbon count {carbon_count} outside typical diterpenoid range (18-22)"

    # Check for cyclic structure
    ring_info = mol.GetRingInfo()
    if not ring_info.NumRings():
        return False, "No ring structures found"

    # Check for methyl groups
    methyl_count = len(mol.GetSubstructMatches(Chem.MolFromSmarts('C[CH3]')))
    if methyl_count < 2:
        return False, "Too few methyl groups"

    # Calculate number of sp2 carbons (for unsaturation)
    sp2_carbons = len([atom for atom in mol.GetAtoms() if 
                      atom.GetSymbol() == 'C' and 
                      atom.GetHybridization() == Chem.HybridizationType.SP2])
    
    # Calculate number of oxygen atoms (for oxidation)
    oxygen_count = len([atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'O'])

    # Typical diterpenoid characteristics found
    reasons = []
    if sp2_carbons > 0:
        reasons.append(f"Contains {sp2_carbons} unsaturated carbons")
    if oxygen_count > 0:
        reasons.append(f"Contains {oxygen_count} oxygen atoms")
    reasons.append(f"Contains {methyl_count} methyl groups")
    reasons.append(f"Contains {ring_info.NumRings()} rings")
    reasons.append(f"Contains {carbon_count} carbons")

    return True, "; ".join(reasons)


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23849',
                          'name': 'diterpenoid',
                          'definition': 'Any terpenoid derived from a '
                                        'diterpene. The term includes '
                                        'compounds in which the C20 skeleton '
                                        'of the parent diterpene has been '
                                        'rearranged or modified by the removal '
                                        'of one or more skeletal atoms '
                                        '(generally methyl groups).',
                          'parents': ['CHEBI:26873']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 57,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 100,
    'num_negatives': None,
    'precision': 1.0,
    'recall': 0.3630573248407643,
    'f1': 0.5327102803738317,
    'accuracy': 0.3630573248407643}


"""
Classifies: CHEBI:17021 S-substituted glutathione
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_S_substituted_glutathione(smiles: str):
    """
    Determines if a molecule is an S-substituted glutathione, which is a glutathione derivative
    where the hydrogen of the thiol has been replaced by any other group.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is an S-substituted glutathione, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Get the SMARTS pattern for glutathione
    glutathione_smarts = "[NX3H2,NX4H3+]-[C@@H](C(=O)N[C@@H](C(=O)O)CS[#6])-C(=O)O"

    # Match the SMARTS pattern against the molecule
    matches = mol.GetSubstructMatches(Chem.MolFromSmarts(glutathione_smarts))

    if not matches:
        return False, "Molecule does not contain the glutathione substructure"

    # Check if the sulfur atom has a non-hydrogen substituent
    for match in matches:
        s_atom_idx = match[-1]
        s_atom = mol.GetAtomWithIdx(s_atom_idx)
        for neighbor_atom in s_atom.GetNeighbors():
            if neighbor_atom.GetSymbol() != "H":
                return True, "Molecule is an S-substituted glutathione"

    return False, "Molecule is glutathione or a non-S-substituted glutathione derivative"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17021',
                          'name': 'S-substituted glutathione',
                          'definition': 'A glutathione derivative that is '
                                        'glutathione in which the hydrogen of '
                                        'the thiol has been replaced by any '
                                        'other group.',
                          'parents': ['CHEBI:24337']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183917,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945627942888}


"""
Classifies: CHEBI:24396 glycopeptide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_glycopeptide(smiles: str):
    """
    Determines if a molecule is a glycopeptide (peptide with glycan moieties attached).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a glycopeptide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for peptide backbone
    peptide_pattern = Chem.MolFromSmarts('[NX3][CX4H1][CX3](=[OX1])[NX3]')
    if not mol.HasSubstructMatch(peptide_pattern):
        return False, "No peptide backbone found"
        
    # Check for glycan moieties
    # Look for pyranose rings with multiple OH groups
    glycan_pattern = Chem.MolFromSmarts('[CR1]1[CR1][CR1][CR1][CR1][OR1]1')
    if not mol.HasSubstructMatch(glycan_pattern):
        return False, "No glycan moieties found"
        
    # Check for glycosidic linkages
    glycosidic_pattern = Chem.MolFromSmarts('[OR1][CR1]1O[CR1][CR1][CR1][CR1][CR1]1')
    if not mol.HasSubstructMatch(glycosidic_pattern):
        return False, "No glycosidic linkages found"

    # Count number of amino acids (peptide bonds)
    peptide_matches = len(mol.GetSubstructMatches(peptide_pattern))
    
    # Count number of glycan rings
    glycan_matches = len(mol.GetSubstructMatches(glycan_pattern))
    
    # Count glycosidic linkages 
    glycosidic_matches = len(mol.GetSubstructMatches(glycosidic_pattern))

    return True, f"Found {peptide_matches} peptide bonds, {glycan_matches} glycan rings, and {glycosidic_matches} glycosidic linkages"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24396',
                          'name': 'glycopeptide',
                          'definition': 'Any carbohydrate derivative that '
                                        'consists of glycan moieties '
                                        'covalently attached to the side '
                                        'chains of the amino acid residues '
                                        'that constitute the peptide.',
                          'parents': ['CHEBI:16670', 'CHEBI:63299']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183843,
    'num_false_negatives': 9,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999510475817506}


"""
Classifies: CHEBI:183508 methyl-branched fatty acyl-CoA(4-)
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprint
from rdkit.Chem import Descriptors

def is_methyl_branched_fatty_acyl_CoA_4__(smiles: str):
    """
    Determines if a molecule is a methyl-branched fatty acyl-CoA(4-).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a methyl-branched fatty acyl-CoA(4-), False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for required substructures
    coenzyme_a = Chem.MolFromSmarts('[O-]P([O-])(=O)OCC1OC(n2cnc3c(N)ncnc32)C(O)C1OP([O-])([O-])=O')
    phosphate = Chem.MolFromSmarts('OP([O-])([O-])=O')
    thioester = Chem.MolFromSmarts('SC(=O)')
    
    if not mol.HasSubstructMatch(coenzyme_a):
        return False, "Missing coenzyme A structure"
    
    if len(mol.GetSubstructMatches(phosphate)) < 3:
        return False, "Missing required phosphate groups"
        
    if not mol.HasSubstructMatch(thioester):
        return False, "Missing thioester linkage"

    # Check for methyl branching
    methyl_branch = Chem.MolFromSmarts('CC(C)')
    if not mol.HasSubstructMatch(methyl_branch):
        return False, "No methyl branching found"

    # Check for fatty acyl chain
    carbon_chain = Chem.MolFromSmarts('CCCCC')
    if not mol.HasSubstructMatch(carbon_chain):
        return False, "Missing fatty acyl chain"

    # Count formal charges to verify 4- charge state
    total_charge = sum([atom.GetFormalCharge() for atom in mol.GetAtoms()])
    if total_charge != -4:
        return False, f"Total charge is {total_charge}, not -4"

    return True, "Methyl-branched fatty acyl-CoA(4-) structure confirmed"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:183508',
                          'name': 'methyl-branched fatty acyl-CoA(4-)',
                          'definition': 'A branched-chain fatty acyl-CoA(4-) '
                                        'arising from deprotonation of the '
                                        'phosphate and diphosphate OH groups '
                                        'of any methyl-branched-chain fatty '
                                        'acyl-CoA; major species at pH 7.3.',
                          'parents': ['CHEBI:183509']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183875,
    'num_false_negatives': 6,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999673702013803}


"""
Classifies: CHEBI:25905 peptide hormone
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_peptide_hormone(smiles: str):
    """
    Determines if a molecule is a peptide hormone based on structural characteristics.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is likely a peptide hormone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check molecular weight - peptide hormones are typically >500 Da
    mol_weight = Descriptors.ExactMolWt(mol)
    if mol_weight < 500:
        return False, "Molecular weight too low for peptide hormone"

    # Count peptide bonds (-C(=O)-N-)
    peptide_bond_pattern = Chem.MolFromSmarts('[C](=O)-[NH]')
    peptide_bonds = len(mol.GetSubstructMatches(peptide_bond_pattern))
    if peptide_bonds < 3:  # Minimum 3 peptide bonds (tetrapeptide)
        return False, "Too few peptide bonds"

    # Check for presence of amino acids
    amino_acids = 0
    
    # Common amino acid side chains patterns
    aa_patterns = [
        '[NH2]-[CH]-C(=O)',  # Basic amino acid structure
        'CC(C)C[CH]',  # Leu
        'CC[CH](C)',  # Ile
        'CC[CH]',  # Val
        'C1=CC=C(C=C1)C[CH]',  # Phe
        'C1=CC=C(O)C=C1C[CH]',  # Tyr
        'C1=CNC=N1C[CH]',  # His
        'NCCCCC[CH]',  # Lys
        'NC(=N)NCCC[CH]',  # Arg
        'SCC[CH]',  # Met
        'OCC[CH]',  # Ser
        'O[CH](C)C[CH]',  # Thr
        'NCC(=O)',  # Gly
        'CC[CH]',  # Ala
        'N[CH]CC(=O)',  # Asn
        'N[CH]CCC(=O)',  # Gln
        'OC(=O)CC[CH]',  # Glu
        'OC(=O)C[CH]',  # Asp
        'C1=CNc2c1cccc2C[CH]',  # Trp
        'SCC[CH]'  # Cys
    ]

    for pattern in aa_patterns:
        substructure = Chem.MolFromSmarts(pattern)
        if substructure:
            matches = mol.GetSubstructMatches(substructure)
            amino_acids += len(matches)

    if amino_acids < 3:  # Minimum 3 amino acid residues
        return False, "Too few amino acid residues detected"

    # Check for basic characteristics that suggest bioactivity
    # - Presence of basic residues (Lys, Arg) often important for receptor binding
    basic_residues = Chem.MolFromSmarts('NC(=N)N') # Arg guanidino group
    lys_pattern = Chem.MolFromSmarts('NCCCCC')
    
    has_basic = False
    if basic_residues and mol.HasSubstructMatch(basic_residues):
        has_basic = True
    if lys_pattern and mol.HasSubstructMatch(lys_pattern):
        has_basic = True

    # Calculate rotatable bonds - peptide hormones typically flexible
    rot_bonds = rdMolDescriptors.CalcNumRotatableBonds(mol)
    
    if peptide_bonds >= 3 and amino_acids >= 3 and has_basic and rot_bonds >= 10:
        return True, f"Likely peptide hormone: contains {peptide_bonds} peptide bonds, {amino_acids} amino acid residues, basic residues, and appropriate flexibility"
    elif peptide_bonds >= 3 and amino_acids >= 3:
        return True, f"Possible peptide hormone: contains {peptide_bonds} peptide bonds and {amino_acids} amino acid residues but lacks typical basic residues"
    else:
        return False, "Structure lacks characteristics of peptide hormones"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25905',
                          'name': 'peptide hormone',
                          'definition': 'Any peptide with hormonal activity in '
                                        'animals, whether endocrine, '
                                        'neuroendocrine, or paracrine.',
                          'parents': ['CHEBI:16670']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 100,
    'num_true_negatives': 1406,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0196078431372549,
    'recall': 1.0,
    'f1': 0.038461538461538464,
    'accuracy': 0.9336870026525199}


"""
Classifies: CHEBI:27175 tyramines
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_tyramines(smiles: str):
    """
    Determines if a molecule is a tyramine (aralkylamino compound containing a tyramine skeleton).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a tyramine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of the tyramine substructure
    tyramine_substructure = Chem.MolFromSmarts('c1ccc(cc1)CCNC')
    matches = mol.GetSubstructMatches(tyramine_substructure)

    if not matches:
        return False, "Tyramine substructure not found"

    # Check if the matched substructure is aromatic and connected to an amino group
    for match in matches:
        atoms = [mol.GetAtomWithIdx(idx) for idx in match]
        aromatic_atoms = [atom for atom in atoms[:4] if atom.GetIsAromatic()]
        if len(aromatic_atoms) == 4:
            amino_group = atoms[-1].GetNeighbors()
            if any(atom.GetSymbol() == 'N' and atom.GetTotalNumHs() < 2 for atom in amino_group):
                return True, "Tyramine substructure found"

    return False, "Tyramine substructure not correctly substituted"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:27175',
                          'name': 'tyramines',
                          'definition': 'Aralkylamino compounds which contain '
                                        'a tyramine skeleton.',
                          'parents': ['CHEBI:33853', 'CHEBI:64365']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               "False negatives: [('C1=CC(=CC=C1O)CCNC(CCCCC)=O', 'Tyramine "
               "substructure not correctly substituted')]",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 1187,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.922360248447205}


"""
Classifies: CHEBI:197527 tricosanol
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors

def is_tricosanol(smiles: str):
    """
    Determines if a molecule is a tricosanol (a fatty alcohol with a hydroxy function
    at any position of an unbranched saturated chain of twenty-three carbon atoms).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a tricosanol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for a single hydroxyl group
    hydroxy_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O')
    if hydroxy_count != 1:
        return False, "Molecule does not contain exactly one hydroxyl group"

    # Check for an unbranched alkyl chain of 23 carbon atoms
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count != 23:
        return False, "Molecule does not contain an alkyl chain of 23 carbon atoms"

    # Check if the alkyl chain is unbranched and saturated
    if rdMolDescriptors.CalcNumAliphaticRings(mol) > 0:
        return False, "Molecule contains a cyclic structure"
    if rdMolDescriptors.CalcNumAromaticRings(mol) > 0:
        return False, "Molecule contains an aromatic ring"
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            if atom.GetDegree() > 4:
                return False, "Molecule contains a branched alkyl chain"
            elif atom.GetDegree() == 4:
                neighbors = [n.GetSymbol() for n in atom.GetNeighbors()]
                if 'O' not in neighbors:
                    return False, "Molecule contains a branched alkyl chain"

    # Check if the hydroxyl group is attached to the carbon chain
    hydroxy_atom = next(atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'O')
    if hydroxy_atom.GetDegree() != 1:
        return False, "Hydroxyl group is not attached to the alkyl chain"

    return True, "Molecule is a tricosanol"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:197527',
                          'name': 'tricosanol',
                          'definition': 'A fatty alcohol consisting of a '
                                        'hydroxy function at any position of '
                                        'an unbranched saturated chain of '
                                        'twenty-three carbon atoms.',
                          'parents': [   'CHEBI:134179',
                                         'CHEBI:197528',
                                         'CHEBI:50584']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               "False negatives: [('CCCCCCCCCCCCCCC(O)CCCCCCCC', 'Molecule "
               "contains a branched alkyl chain')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 6,
    'num_true_negatives': 183917,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.14285714285714285,
    'recall': 1.0,
    'f1': 0.25,
    'accuracy': 0.9999673778299732}


"""
Classifies: CHEBI:134363 tertiary amine oxide
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_tertiary_amine_oxide(smiles: str):
    """
    Determines if a molecule is a tertiary amine oxide.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a tertiary amine oxide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
    
    # Find N-oxide substructure
    # Pattern matches N+ with exactly 4 bonds, one to O-, and three to C
    n_oxide_pattern = Chem.MolFromSmarts('[NX4+;$([N](-[OX1-])(-[#6])(-[#6])(-[#6]))]')
    
    if not mol.HasSubstructMatch(n_oxide_pattern):
        return False, "No tertiary amine oxide group found"
    
    # Check for presence of N-oxide group that's not part of N-oxide functionality
    n_oxide_false_pos = Chem.MolFromSmarts('[N]-[O]')
    matches_false_pos = mol.GetSubstructMatches(n_oxide_false_pos)
    matches_true = mol.GetSubstructMatches(n_oxide_pattern)
    
    # If we have N-O bonds that aren't part of our N-oxide pattern, it might be a different type
    if len(matches_false_pos) > len(matches_true):
        return False, "Contains N-O bond but not in tertiary amine oxide configuration"
        
    # Check each match to verify it's a proper tertiary amine oxide
    for match in matches_true:
        n_idx = match[0]  # Index of nitrogen atom
        n_atom = mol.GetAtomWithIdx(n_idx)
        
        # Verify formal charge is +1
        if n_atom.GetFormalCharge() != 1:
            continue
            
        # Count number of carbons directly bonded to nitrogen
        carbon_count = sum(1 for neighbor in n_atom.GetNeighbors() 
                         if neighbor.GetAtomicNum() == 6)
        
        # Must have exactly 3 carbons bonded to the nitrogen
        if carbon_count != 3:
            continue
            
        # Count number of oxygens with -1 charge bonded to nitrogen
        oxide_count = sum(1 for neighbor in n_atom.GetNeighbors() 
                        if neighbor.GetAtomicNum() == 8 and 
                        neighbor.GetFormalCharge() == -1)
        
        # Must have exactly one oxide oxygen
        if oxide_count != 1:
            continue
            
        return True, "Tertiary amine oxide found"
    
    return False, "No valid tertiary amine oxide group found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:134363',
                          'name': 'tertiary amine oxide',
                          'definition': 'An N-oxide where there are three '
                                        'organic groups bonded to the nitrogen '
                                        'atom.',
                          'parents': ['CHEBI:35580']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.1111111111111111 is too low.\n'
               'True positives: '
               "[('C[N+]1(CCC2=CC(=C(C=C2[C@@H]1CC3=CC(=C(C=C3)OC)O)O)OC)[O-]', "
               "'Tertiary amine oxide found'), "
               "('[C@@]12([N+]3(CC[C@H]1OC(/C(/CC(=C)[C@@](C(OCC2=CC3)=O)(C)O)=C/C)=O)[O-])[H]', "
               "'Tertiary amine oxide found'), "
               "('C(CCCCCCCC)CCC[N+](C)([O-])C', 'Tertiary amine oxide "
               "found'), "
               "('[C@@]12([N+]3(CC[C@H]1OC(/C(/CC(=C)[C@](C(OCC2=CC3)=O)(OC(=O)C)C)=C\\\\C)=O)[O-])[H]', "
               "'Tertiary amine oxide found')]\n"
               'False positives: '
               "[('O=C1N([C@]23[C@H](C(C)(C)[C@@]4(C2)C(=O)NC5=C4C=CC6=C5C(=O)CC(O6)(C)C)C[C@]17[N+]([O-])(C[C@@H](C7)C)C3)C', "
               "'Tertiary amine oxide found'), "
               "('C(CC[N+](C)(C)[O-])=C1C=2C(CCC3=C1C=CC=C3)=CC=CC2', "
               "'Tertiary amine oxide found'), "
               "('S(=O)(=O)(N1CC[N+]([O-])(CC1)C)C2=CC(=C(OCC)C=C2)C=3NC(=O)C=4N(N=C(C4N3)CCC)C', "
               "'Tertiary amine oxide found'), "
               "('[O-][N+]1(C2C=3C(CC1)=CC=4OCOC4C3C=5C(C2O)=CC=CC5)C', "
               "'Tertiary amine oxide found'), "
               "('[O-][N+]1(C2CCC1CC(OC(=O)/C(/C)=C/C)C2)C', 'Tertiary amine "
               "oxide found'), "
               "('O=C1N([C@]23[C@H](C(C)(C)[C@@]4(C2)C(=O)NC5=C4C=CC6=C5OC=CC(O6)(C)C)C[C@]17[N+]([O-])(CC[C@]7(O)C)C3)C', "
               "'Tertiary amine oxide found'), "
               "('[O-][N+]12C(C(CC1)COC(=O)C(O)(C(C)C)C(O)C)CCC2', 'Tertiary "
               "amine oxide found'), "
               "('O=C(NC1=C(C=CC=C1C)C)C[N+]([O-])(CC)CC', 'Tertiary amine "
               "oxide found'), "
               "('[O-][N+]12C(C(O)CC1)C(=CC2)COC(=O)C(O)(C(C)C)C(O)C', "
               "'Tertiary amine oxide found'), "
               "('O[C@]1([C@H](CCCC1)C[N+]([O-])(C)C)C2=CC(OC)=CC=C2', "
               "'Tertiary amine oxide found'), "
               "('S(=O)(=O)(C1=C(N)C=C(OC)C(=C1)C(=O)NCC2[N+]([O-])(CCC2)CC)CC', "
               "'Tertiary amine oxide found'), "
               "('O=C(N(O)CCC[C@H]([N+]([O-])(C)C)C(=O)O)/C=C(/CCOC1OC(C(OC)C(C1O)O)CO)\\\\C', "
               "'Tertiary amine oxide found'), "
               "('O[C@@]12C34N(=O)(CCC1)CCC[C@]3(O)C(=O)C[C@@]2(C[C@](C4)(C)[H])[H]', "
               "'Tertiary amine oxide found'), "
               "('[O-][N+]1([C@@]2(C=3C(CC1)=CC(OC)=C(OC)C3OC=4C=C5C(N(CCC5=CC4OC)C)CC6=CC=C(OC=7C=C(C2)C=CC7OC)C=C6)[H])C', "
               "'Tertiary amine oxide found'), "
               "('ClC=1C(NC(=O)C=2SC(NC=3N=C(N=C(N4CC[N+]([O-])(CC4)CCO)C3)C)=NC2)=C(C=CC1)C', "
               "'Tertiary amine oxide found'), "
               "('O=C1C2=C(O)C(=CC=C2C(=O)C3=C1[C@H](O[C@H]4[C@@H]3OC(=O)C4)C)[C@@H]5O[C@@H]([C@@H](O)[C@@H](C5)[N+]([O-])(C)C)C', "
               "'Tertiary amine oxide found'), "
               "('C[N+]1([C@H]2C[C@@H](C[C@@H]1CC2)OC(C(CO)C3=CC=CC=C3)=O)[O-]', "
               "'Tertiary amine oxide found'), "
               "('O(CC[N+]([O-])(C)C)C(=O)C1=CC=C(NCCCC)C=C1', 'Tertiary amine "
               "oxide found'), "
               "('ClC1=CN=C(N2C(=O)C=3N=CC=NC3[C@@H]2OC(=O)N4CC[N+]([O-])(C)CC4)C=C1', "
               "'Tertiary amine oxide found'), "
               "('[O-][N+]1(CCN(CC1)C(=O)N(CC)CC)C', 'Tertiary amine oxide "
               "found'), ('O=C1N2C(C3C4[N+]([O-])(CCC3)CCCC4C2)CC=C1', "
               "'Tertiary amine oxide found'), "
               "('[O-][N@+]12[C@]34N5C(=O)C[C@]3(CC[C@]4([C@](CC1)(C=6C5=CC(OC)=C(OC)C6)[H])[H])C=CC2', "
               "'Tertiary amine oxide found'), "
               "('O=C1N[C@H](C(=O)N[C@H]1CCCN(O)C(=O)/C=C(/CCOC(=O)[C@@H]([N+]([O-])(C)C)CCCN(O)C(=O)/C=C(/CCO)\\\\C)\\\\C)CCCN(O)C(=O)/C=C(/CCO[C@@H]2O[C@@H]([C@@H](O)[C@@H]([C@@H]2O)O)CO)\\\\C', "
               "'Tertiary amine oxide found'), "
               "('[O-][N+]12CC(CCC1)(CCC3=NC=4C(C3(O)CC2)=CC=CC4)CC', "
               "'Tertiary amine oxide found'), "
               "('[O-][N+](CCC=1C2=C(NC1)C=CC(OC)=C2)(C)C', 'Tertiary amine "
               "oxide found'), ('C(C[N+](CCCl)([O-])C)Cl.Cl', 'Tertiary amine "
               "oxide found'), "
               "('S1C=CNC(=O)[C@@H](NC([C@@H](NC([C@H](C1)NC(=O)[C@H](NC(=O)/C(/NC(=O)[C@@H](NC(=O)CNC(=O)[C@H](NC(=O)[C@H](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@H](NC(=O)[C@H](NC(=O)[C@H](NC(=O)/C(/NC(=O)[C@@H]2N(C(=O)/C(/NC(=O)[C@H](NC(=O)[C@@H]3N(C(=O)/C(/NC(=O)[C@@H]([N+]([O-])(C)C)C)=C/C)CCC3)C)=C/C)CCC2)=C/C)C)C)CCC(=O)N)CC4=CC=CC=C4)C(C)C)[C@@H](CC)C)CCC(=O)N)CO)=C/C)[C@@H](CC)C)=O)CC(C)C)=O)C(C)C', "
               "'Tertiary amine oxide found'), "
               "('C(CC[N+](C)(C)[O-])N1C=2C(CCC3=C1C=CC=C3)=CC=CC2', 'Tertiary "
               "amine oxide found'), "
               "('[O-][N+]12C([C@@H](CC1)COC(=O)C(O)(C(C)C)C(O)C)CCC2', "
               "'Tertiary amine oxide found'), "
               "('C\\\\C=C1\\\\C[C@@H](C)[C@](O)(CO)C(=O)OCC2=CCN3(=O)CC[C@@H](OC1=O)[C@@H]23', "
               "'Tertiary amine oxide found'), "
               "('O1[C@@]2([C@]34[C@](O)([C@]([N@@+]([O-])(CC3)C)(CC5=C4C1=C(OC)C=C5)[H])CC[C@@H]2O)[H]', "
               "'Tertiary amine oxide found'), ('CN(=O)(CCCl)CCCl', 'Tertiary "
               "amine oxide found'), "
               "('[O-][N+]1(CC2N(C3=NC=CC=C3CC4=C2C=CC=C4)CC1)C', 'Tertiary "
               "amine oxide found'), "
               "('O=C1N[C@H](C(=O)N[C@H]1CCCN(O)C(=O)/C=C(/CCOC(=O)[C@@H]([N+]([O-])(C)C)CCCN(O)C(=O)/C=C(/CCO[C@@H]2O[C@@H]([C@@H](O)[C@@H]([C@@H]2O)O)CO)\\\\C)\\\\C)CCCN(O)C(=O)/C=C(/CCO[C@@H]3O[C@@H]([C@@H](O)[C@@H]([C@@H]3O)O)CO)\\\\C', "
               "'Tertiary amine oxide found'), "
               "('OC1C2([N+]([O-])(CCC=3C2=CC(O)=C(OC)C3)C)CC4=C1C=5OCOC5C=C4', "
               "'Tertiary amine oxide found'), "
               "('O=C(N1[C@@H](C(=O)N[C@H](C(=O)N[C@@H](C(=O)NC(C(=O)N[C@H](C(=O)N[C@H](C(=O)NC(C(=O)NC(C(=O)NCCC(=O)N[C@H](C[N+]([O-])(C)C)C)(C)C)(C)C)CC(C)C)CC(C)C)(C)C)[C@@H](O)C(C)C)CC(C)C)C[C@@H](C1)C)/C=C\\\\[C@@H](CC)C', "
               "'Tertiary amine oxide found'), "
               "('ClC1=CC=2N(CCC[N+]([O-])(C)C)C=3C(CCC2C=C1)=CC=CC3', "
               "'Tertiary amine oxide found'), "
               "('OC1(C(C[N+]([O-])(C)C)C2=CC=C(OC)C=C2)CCCCC1', 'Tertiary "
               "amine oxide found'), "
               "('O=C(NC=CC1=CC=CC=C1)C2[N+]([O-])(CCCC2)C', 'Tertiary amine "
               "oxide found'), "
               "('O=C1N[C@H](C(=O)N[C@H]1CCCN(O)C(=O)/C=C(/CCOC(=O)[C@@H]([N+]([O-])(C)C)CCCN(O)C(=O)/C=C(/CCO[C@@H]2O[C@@H]([C@@H](OC)[C@@H]([C@@H]2O)O)CO)\\\\C)\\\\C)CCCN(O)C(=O)/C=C(/CCO[C@@H]3O[C@@H]([C@@H](OC)[C@@H]([C@@H]3O)O)CO)\\\\C', "
               "'Tertiary amine oxide found'), "
               "('ClC1=CC=2N=C(N3CC[N+]([O-])(CC3)C)C=4C(NC2C=C1)=CC=CC4', "
               "'Tertiary amine oxide found'), ('[N+]1(CCOCC1)(C)[O-]', "
               "'Tertiary amine oxide found'), "
               "('[O-][N+](CCC(C1=CC=CC=C1)C=2N=CC=CC2)(C)C', 'Tertiary amine "
               "oxide found'), "
               "('O1[C@@]2([C@]34[C@]([C@]([N@@+]([O-])(CC3)C)(CC5=C4C1=C(OC)C=C5)[H])(C=C[C@@H]2O)[H])[H]', "
               "'Tertiary amine oxide found'), "
               "('O=C(N1[C@H](C(=O)N[C@@H](C(=O)N[C@@H](C(=O)NC(C(=O)N[C@H](C(=O)N[C@H](C(=O)NC(C(=O)NC(C(=O)NCCC(=O)N[C@H](C[N+]([O-])(C)C)C)(C)C)(C)C)CC(C)C)CC(C)C)(C)C)[C@H](O)C(C)C)C[C@H](C[C@H](O)CC(=O)CC)C)C[C@@H](C1)C)/C=C/[C@@H](CC)C', "
               "'Tertiary amine oxide found'), "
               "('CN(C)(=O)CCCC1(OCc2cc(ccc12)C#N)c1ccc(F)cc1', 'Tertiary "
               "amine oxide found'), "
               "('FC1=C(N2CC[N+]([O-])(CC2)C)C=3OCC(N4C3C(=C1)C(=O)C(=C4)C(O)=O)C', "
               "'Tertiary amine oxide found'), "
               "('S(=O)(=O)(N1CC[N+]([O-])(CC1)CC)C2=CC(=C(OCC)C=C2)C=3NC(=O)C=4N(N3)C(=NC4C)CCC', "
               "'Tertiary amine oxide found'), "
               "('O=C1OC(C(O)(C(O)C(C(=O)C(C)CC(C(C(C(C1C)OC2OC(C(O)C(C2)(OC)C)C)C)OC3OC(CC(C3O)[N+]([O-])(C)C)C)(O)C)C)C)CC', "
               "'Tertiary amine oxide found'), "
               "('[O-][N+]1(CCN(CC1)CC2=CC=C(C=C2)C(=O)NC3=CC(NC=4N=C(C=5C=CC=NC5)C=CN4)=C(C=C3)C)C', "
               "'Tertiary amine oxide found'), "
               "('O1CC=2C(/C(=C\\\\CC[N+]([O-])(C)C)/C=3C1=CC=CC3)=CC=CC2', "
               "'Tertiary amine oxide found'), "
               "('C=1[C@H]([C@H](OC([C@@H]([C@H]([C@H](C[C@H](C(C1)=O)C)C)OC2O[C@@H](C[C@@H]([C@H]2O)[N+](C)(C)[O-])C)C)=O)C(C)=O)C', "
               "'Tertiary amine oxide found'), "
               "('S(=O)(=O)(CCC1=CC=2C(C[C@@H]3[N+]([O-])(CCC3)C)=CNC2C=C1)C4=CC=CC=C4', "
               "'Tertiary amine oxide found'), "
               "('O([C@@H]1[C@@]2([N+]([O-])(CC1)CC=C2COC(=O)[C@@](O)(C(O)(C)C)[C@@H](O)C)[H])C(=O)/C(/C)=C\\\\C', "
               "'Tertiary amine oxide found'), "
               "('[O-][N+](CCCCCCCCCC)(CCCCCCCCCC)C', 'Tertiary amine oxide "
               "found'), ('CNC(NCCSCc1ccc(CN(C)(C)=O)o1)=C[N+]([O-])=O', "
               "'Tertiary amine oxide found'), "
               "('C(CC[N@@+]1([O-])CC[C@](CC1)(O)C2=CC=C(C=C2)Cl)(C(N(C)C)=O)(C3=CC=CC=C3)C4=CC=CC=C4', "
               "'Tertiary amine oxide found'), "
               "('[O-][N+]12[C@@]([C@@H](O)CC1)(C(=CC2)COC(=O)[C@](O)(C(C)C)[C@H](OC)C)[H]', "
               "'Tertiary amine oxide found'), "
               "('ClC1=CC=C(C(N2CC[N+]([O-])(CC2)CCOCC(O)=O)C3=CC=CC=C3)C=C1', "
               "'Tertiary amine oxide found'), "
               "('C/C=C\\\\1/C[N+]2(CC[C@@]34CC=5C=C6C(=CC5O[C@@]74[C@@]2(C[C@@]1([C@@](C(=O)OC)(N7C8=CC=CC=C83)[H])[H])[H])N[C@]9%10CC[C@@]%11(CC)CCC[N@+]%10(CC[C@@]69O)C%11)[O-]', "
               "'Tertiary amine oxide found'), "
               "('FC1=C(N2CCN(CC2)C)C=C3[N+]([O-])(CC)C=C(C(=O)C3=C1)C(O)=O', "
               "'Tertiary amine oxide found'), "
               "('[O-][N@@+]12[C@@]3([C@H]([C@@](C[C@]1(C=4NC=5C(C4C3)=CC=CC5)[H])(/C(/C2)=C\\\\C)[H])C(OC)=O)[H]', "
               "'Tertiary amine oxide found'), "
               "('C/C=C\\\\1/C2CC[N@+]3(CC[C@]4(C5=CC=CC=C5NC4=C2C(=O)OC)[C@@]13[H])[O-]', "
               "'Tertiary amine oxide found'), "
               "('COC([C@@H]1C[C@@]23CCC[N+]4(CCC5(C6=CC=CC=C6N(C(=O)OC)[C@]15CC2)[C@]34[H])[O-])=O', "
               "'Tertiary amine oxide found')]\n"
               'False negatives: []',
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 59,
    'num_true_negatives': 183830,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.06349206349206349,
    'recall': 1.0,
    'f1': 0.11940298507462686,
    'accuracy': 0.9996791612513799}


"""
Classifies: CHEBI:25553 nitrobenzoic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_nitrobenzoic_acid(smiles: str):
    """
    Determines if a molecule is a nitrobenzoic acid (benzoic acid with at least one nitro substituent).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a nitrobenzoic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for benzoic acid substructure
    benzoic_acid_pattern = Chem.MolFromSmarts('c1ccccc1C(=O)O')
    if not mol.HasSubstructMatch(benzoic_acid_pattern):
        return False, "No benzoic acid substructure found"

    # Check for nitro group substructure
    nitro_pattern = Chem.MolFromSmarts('[N+](=O)[O-]')
    if not mol.HasSubstructMatch(nitro_pattern):
        return False, "No nitro group found"

    # Get all benzoic acid matches
    benzoic_matches = mol.GetSubstructMatches(benzoic_acid_pattern)
    
    # Get all nitro group matches
    nitro_matches = mol.GetSubstructMatches(nitro_pattern)

    # For each benzoic acid match, check if there's a nitro group attached to the ring
    for benzoic_match in benzoic_matches:
        ring_atoms = set(benzoic_match[:6])  # First 6 atoms are the benzene ring
        
        for nitro_match in nitro_matches:
            nitro_n = nitro_match[0]  # nitrogen atom of nitro group
            
            # Get the atom the nitro group is attached to
            nitro_neighbors = [x.GetIdx() for x in mol.GetAtomWithIdx(nitro_n).GetNeighbors() 
                             if x.GetIdx() not in nitro_match]
            
            # If any of the nitro group's neighbors are in the benzene ring
            if any(neighbor in ring_atoms for neighbor in nitro_neighbors):
                # Count number of nitro groups attached to this ring
                nitro_count = sum(1 for nitro_m in nitro_matches 
                                if any(n in ring_atoms for n in [x.GetIdx() for x in mol.GetAtomWithIdx(nitro_m[0]).GetNeighbors() 
                                                               if x.GetIdx() not in nitro_m]))
                
                return True, f"Found benzoic acid with {nitro_count} nitro group(s) attached to the ring"

    return False, "No nitro group attached to benzoic acid ring"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25553',
                          'name': 'nitrobenzoic acid',
                          'definition': 'Any member of the class of benzoic '
                                        'acids with at least one nitro '
                                        'substituent attached to the benzene '
                                        'ring.',
                          'parents': ['CHEBI:22723', 'CHEBI:35716']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 38,
    'num_true_negatives': 183859,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.07317073170731707,
    'recall': 1.0,
    'f1': 0.13636363636363635,
    'accuracy': 0.9997933659597608}